from collections.abc import Callable
from logging import Logger
from typing import Literal, TypeAlias

from .amqp_object import AMQPObject

AMQPValue: TypeAlias = type[AMQPObject] | AMQPObject | int | str

LOGGER: Logger

def name_or_value(value: AMQPValue) -> str: ...
def sanitize_prefix(function): ...
def check_for_prefix_and_key(function): ...

class CallbackManager:
    CALLS: str
    ARGUMENTS: str
    DUPLICATE_WARNING: str
    CALLBACK: str
    ONE_SHOT: str
    ONLY_CALLER: str
    def __init__(self) -> None: ...
    def add(
        self,
        prefix: str | int,
        key: AMQPValue,
        # Parameter type must match arguments passed to process()
        callback: Callable[..., object],
        one_shot: bool = True,
        only_caller: object | None = None,
        arguments=None,
    ) -> tuple[str | int, str | object]: ...
    def clear(self) -> None: ...
    def cleanup(self, prefix: str | int) -> bool: ...
    def pending(self, prefix: str | int, key: str | object) -> int | None: ...
    def process(self, prefix: str | int, key: AMQPValue, caller, *args, **keywords) -> bool: ...
    def remove(
        self, prefix: str | int, key: AMQPValue, callback_value: Callable[..., object] | None = None, arguments=None
    ) -> Literal[True]: ...
    def remove_all(self, prefix: str | int, key: AMQPValue) -> None: ...



import logging
from _typeshed import Incomplete
from collections.abc import Callable

from authlib.oauth1 import AuthorizationServer as _AuthorizationServer, OAuth1Request

log: logging.Logger

class AuthorizationServer(_AuthorizationServer):
    app: Incomplete
    query_client: Incomplete
    token_generator: Incomplete
    def __init__(self, app=None, query_client=None, token_generator=None): ...
    SUPPORTED_SIGNATURE_METHODS: Incomplete
    def init_app(self, app, query_client=None, token_generator=None): ...
    def register_hook(self, name, func) -> None: ...
    def create_token_generator(self, app) -> Callable[[], dict[str, str]]: ...
    def get_client_by_id(self, client_id): ...
    def exists_nonce(self, nonce, request): ...
    def create_temporary_credential(self, request): ...
    def get_temporary_credential(self, request): ...
    def delete_temporary_credential(self, request): ...
    def create_authorization_verifier(self, request): ...
    def create_token_credential(self, request): ...
    def check_authorization_request(self) -> OAuth1Request: ...
    def create_authorization_response(self, request=None, grant_user=None): ...
    def create_token_response(self, request=None): ...
    def create_oauth1_request(self, request) -> OAuth1Request: ...
    def handle_response(self, status_code, payload, headers): ...



import logging
import socket
from collections.abc import Mapping
from datetime import timedelta
from typing import Final

from gunicorn.config import Config
from gunicorn.glogging import Logger
from gunicorn.http import Request
from gunicorn.http.wsgi import Response

from .._types import _EnvironType
from ..glogging import _LogLevelType

METRIC_VAR: Final = "metric"
VALUE_VAR: Final = "value"
MTYPE_VAR: Final = "mtype"
GAUGE_TYPE: Final = "gauge"
COUNTER_TYPE: Final = "counter"
HISTOGRAM_TYPE: Final = "histogram"
TIMER_TYPE: Final = "timer"

class Statsd(Logger):
    prefix: str
    sock: socket.socket | None
    dogstatsd_tags: str | None
    cfg: Config

    def __init__(self, cfg: Config) -> None: ...
    def critical(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def error(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def warning(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def info(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def debug(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def exception(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = True,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def log(
        self,
        lvl: _LogLevelType,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def access(self, resp: Response, req: Request, environ: _EnvironType, request_time: timedelta) -> None: ...
    def gauge(self, name: str, value: float) -> None: ...
    def increment(self, name: str, value: int, sampling_rate: float = 1.0) -> None: ...
    def decrement(self, name: str, value: int, sampling_rate: float = 1.0) -> None: ...
    def timer(self, name: str, value: float) -> None: ...
    def histogram(self, name: str, value: float) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar, NamedTuple

import kafka.errors as Errors

log: Incomplete
READ_UNCOMMITTED: int
READ_COMMITTED: int
ISOLATION_LEVEL_CONFIG: Incomplete

class ConsumerRecord(NamedTuple):
    topic: str
    partition: int
    leader_epoch: int | None
    offset: int
    timestamp: int
    timestamp_type: int
    key: Incomplete
    value: Incomplete
    headers: list[tuple[str, bytes]]
    checksum: int | None
    serialized_key_size: int
    serialized_value_size: int
    serialized_header_size: int

class CompletedFetch(NamedTuple):
    topic_partition: Incomplete
    fetched_offset: Incomplete
    response_version: Incomplete
    partition_data: Incomplete
    metric_aggregator: Incomplete

class ExceptionMetadata(NamedTuple):
    partition: Incomplete
    fetched_offset: Incomplete
    exception: Incomplete

class NoOffsetForPartitionError(Errors.KafkaError): ...
class RecordTooLargeError(Errors.KafkaError): ...

class Fetcher:
    DEFAULT_CONFIG: Incomplete
    config: Incomplete
    def __init__(self, client, subscriptions, **configs) -> None: ...
    def send_fetches(self): ...
    def in_flight_fetches(self): ...
    def reset_offsets_if_needed(self): ...
    def offsets_by_times(self, timestamps, timeout_ms=None): ...
    def beginning_offsets(self, partitions, timeout_ms): ...
    def end_offsets(self, partitions, timeout_ms): ...
    def beginning_or_end_offset(self, partitions, timestamp, timeout_ms): ...
    def fetched_records(self, max_records=None, update_offsets: bool = True): ...
    def close(self) -> None: ...

    class PartitionRecords:
        fetch_offset: Incomplete
        topic_partition: Incomplete
        leader_epoch: int
        next_fetch_offset: Incomplete
        bytes_read: int
        records_read: int
        isolation_level: Incomplete
        aborted_producer_ids: Incomplete
        aborted_transactions: Incomplete
        metric_aggregator: Incomplete
        check_crcs: Incomplete
        record_iterator: Incomplete
        on_drain: Incomplete
        def __init__(
            self,
            fetch_offset,
            tp,
            records,
            key_deserializer=None,
            value_deserializer=None,
            check_crcs: bool = True,
            isolation_level=0,
            aborted_transactions=None,
            metric_aggregator=None,
            on_drain=...,
        ) -> None: ...
        def __bool__(self) -> bool: ...
        __nonzero__ = __bool__
        def drain(self) -> None: ...
        def take(self, n=None): ...

class FetchSessionHandler:
    node_id: Incomplete
    next_metadata: Incomplete
    session_partitions: Incomplete
    def __init__(self, node_id) -> None: ...
    def build_next(self, next_partitions): ...
    def handle_response(self, response): ...
    def handle_error(self, _exception) -> None: ...

class FetchMetadata:
    MAX_EPOCH: int
    INVALID_SESSION_ID: int
    THROTTLED_SESSION_ID: int
    INITIAL_EPOCH: int
    FINAL_EPOCH: int
    INITIAL: ClassVar[FetchMetadata]
    LEGACY: ClassVar[FetchMetadata]
    session_id: Incomplete
    epoch: Incomplete
    def __init__(self, session_id, epoch) -> None: ...
    @property
    def is_full(self): ...
    @classmethod
    def next_epoch(cls, prev_epoch): ...
    def next_close_existing(self): ...
    @classmethod
    def new_incremental(cls, session_id): ...
    def next_incremental(self): ...

class FetchRequestData:
    def __init__(self, to_send, to_forget, metadata) -> None: ...
    @property
    def metadata(self): ...
    @property
    def id(self): ...
    @property
    def epoch(self): ...
    @property
    def to_send(self): ...
    @property
    def to_forget(self): ...

class FetchMetrics:
    total_bytes: int
    total_records: int
    def __init__(self) -> None: ...

class FetchResponseMetricAggregator:
    sensors: Incomplete
    unrecorded_partitions: Incomplete
    fetch_metrics: Incomplete
    topic_fetch_metrics: Incomplete
    def __init__(self, sensors, partitions) -> None: ...
    def record(self, partition, num_bytes, num_records) -> None: ...

class FetchManagerMetrics:
    metrics: Incomplete
    group_name: Incomplete
    bytes_fetched: Incomplete
    records_fetched: Incomplete
    fetch_latency: Incomplete
    records_fetch_lag: Incomplete
    def __init__(self, metrics, prefix) -> None: ...
    def record_topic_fetch_metrics(self, topic, num_bytes, num_records) -> None: ...



from _typeshed import ConvertibleToInt
from collections.abc import Callable, Generator, Iterable, Iterator
from re import Pattern
from typing import Any, ClassVar, Final, Literal

from pygments.filter import Filter
from pygments.lexer import Lexer
from pygments.token import _TokenType

def find_filter_class(filtername: str) -> type[Filter] | None: ...

# Keyword arguments are forwarded to the filter class.
def get_filter_by_name(filtername: str, **options: Any) -> Filter: ...
def get_all_filters() -> Generator[str]: ...

class CodeTagFilter(Filter):
    tag_re: Pattern[str]
    # Arbitrary additional keyword arguments are permitted and are stored in self.options.
    def __init__(
        self, *, codetags: str | list[str] | tuple[str, ...] = ["XXX", "TODO", "FIXME", "BUG", "NOTE"], **options: Any
    ) -> None: ...
    def filter(self, lexer: Lexer | None, stream: Iterable[tuple[_TokenType, str]]) -> Iterator[tuple[_TokenType, str]]: ...

class SymbolFilter(Filter):
    latex_symbols: ClassVar[dict[str, str]]
    isabelle_symbols: ClassVar[dict[str, str]]
    lang_map: ClassVar[dict[Literal["isabelle", "latex"], dict[str, str]]]
    symbols: dict[str, str]  # One of latex_symbols or isabelle_symbols.
    # Arbitrary additional keyword arguments are permitted and are stored in self.options.
    def __init__(self, *, lang: Literal["isabelle", "latex"] = "isabelle", **options: Any) -> None: ...
    def filter(self, lexer: Lexer | None, stream: Iterable[tuple[_TokenType, str]]) -> Iterator[tuple[_TokenType, str]]: ...

class KeywordCaseFilter(Filter):
    convert: Callable[[str], str]
    # Arbitrary additional keyword arguments are permitted and are stored in self.options.
    def __init__(self, *, case: Literal["lower", "upper", "capitalize"] = "lower", **options: Any) -> None: ...
    def filter(self, lexer: Lexer | None, stream: Iterable[tuple[_TokenType, str]]) -> Iterator[tuple[_TokenType, str]]: ...

class NameHighlightFilter(Filter):
    names: set[str]
    tokentype: _TokenType
    # Arbitrary additional keyword arguments are permitted and are stored in self.options.
    def __init__(
        self, *, names: str | list[str] | tuple[str, ...] = [], tokentype: str | _TokenType | None = None, **options: Any
    ) -> None: ...
    def filter(self, lexer: Lexer | None, stream: Iterable[tuple[_TokenType, str]]) -> Iterator[tuple[_TokenType, str]]: ...

class ErrorToken(Exception): ...

class RaiseOnErrorTokenFilter(Filter):
    exception: type[Exception]
    # Arbitrary additional keyword arguments are permitted and are stored in self.options.
    def __init__(self, *, excclass: type[Exception] = ..., **options: Any) -> None: ...
    def filter(self, lexer: Lexer | None, stream: Iterable[tuple[_TokenType, str]]) -> Iterator[tuple[_TokenType, str]]: ...

class VisibleWhitespaceFilter(Filter):
    spaces: str
    tabs: str
    newlines: str
    wstt: bool
    def __init__(
        self,
        *,
        spaces: str | bool = False,
        tabs: str | bool = False,
        newlines: str | bool = False,
        tabsize: ConvertibleToInt = 8,
        wstokentype: bool | int | str = True,  # Any value accepted by get_bool_opt.
        # Arbitrary additional keyword arguments are permitted and are stored in self.options.
        **options: Any,
    ) -> None: ...
    def filter(self, lexer: Lexer | None, stream: Iterable[tuple[_TokenType, str]]) -> Iterator[tuple[_TokenType, str]]: ...

class GobbleFilter(Filter):
    n: int
    # Arbitrary additional keyword arguments are permitted and are stored in self.options.
    def __init__(self, *, n: ConvertibleToInt = 0, **options: Any) -> None: ...
    def gobble(self, value: str, left: int) -> tuple[str, int]: ...
    def filter(self, lexer: Lexer | None, stream: Iterable[tuple[_TokenType, str]]) -> Iterator[tuple[_TokenType, str]]: ...

class TokenMergeFilter(Filter):
    # Arbitrary additional keyword arguments are permitted and are stored in self.options.
    def __init__(self, **options: Any) -> None: ...
    def filter(self, lexer: Lexer | None, stream: Iterable[tuple[_TokenType, str]]) -> Iterator[tuple[_TokenType, str]]: ...

FILTERS: Final[dict[str, type[Filter]]]



import re
from _typeshed import Incomplete
from typing import Final

__version__: Final[str]

def replace(src, sep, rep): ...

keywordsList: list[str]
commentPat: str
pat: str
quotePat: str
tripleQuotePat: str
nonKeyPat: str
keyPat: str
matchPat: str
matchRE: re.Pattern[str]
idKeyPat: str
idRE: re.Pattern[str]

def fontify(pytext, searchfrom: int = 0, searchto=None) -> list[tuple[str, int, int, Incomplete]]: ...
def test(path) -> None: ...



from collections.abc import Generator
from typing import Any

from .resource import Collection, Model

class Plugin(Model):
    @property
    def name(self) -> str | None: ...
    @property
    def enabled(self) -> bool | None: ...
    @property
    def settings(self) -> dict[str, Any] | None: ...
    def configure(self, options: dict[str, Any]) -> None: ...
    def disable(self, force: bool = False) -> None: ...
    def enable(self, timeout: int = 0) -> None: ...
    def push(self) -> Generator[dict[str, Any]]: ...
    def remove(self, force: bool = False) -> bool: ...
    def upgrade(self, remote: str | None = None) -> Generator[dict[str, Any]]: ...

class PluginCollection(Collection[Plugin]):
    model: type[Plugin]
    def create(self, name, plugin_data_dir, gzip: bool = False): ...  # type: ignore[override]
    def get(self, name): ...
    def install(self, remote_name, local_name=None): ...
    def list(self): ...



from _typeshed import Incomplete

from kafka.protocol.api import Request, Response

class BaseApiVersionsResponse(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete
    @classmethod
    def decode(cls, data): ...

class ApiVersionsResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ApiVersionsResponse_v1(BaseApiVersionsResponse):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ApiVersionsResponse_v2(BaseApiVersionsResponse):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ApiVersionsResponse_v3(BaseApiVersionsResponse):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ApiVersionsResponse_v4(BaseApiVersionsResponse):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ApiVersionsRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ApiVersionsResponse_v0
    SCHEMA: Incomplete

class ApiVersionsRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ApiVersionsResponse_v1
    SCHEMA: Incomplete

class ApiVersionsRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ApiVersionsResponse_v2
    SCHEMA: Incomplete

class ApiVersionsRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ApiVersionsResponse_v3
    SCHEMA: Incomplete
    FLEXIBLE_VERSION: bool

class ApiVersionsRequest_v4(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = ApiVersionsResponse_v4
    SCHEMA: Incomplete
    FLEXIBLE_VERSION: bool

ApiVersionsRequest: Incomplete
ApiVersionsResponse: Incomplete



from typing import Any, Generic, TypeVar
from typing_extensions import Self

__tracebackhide__: bool

_V = TypeVar("_V", default=Any)

class ContainsMixin(Generic[_V]):
    def contains(self, *items: object) -> Self: ...
    def does_not_contain(self, *items: object) -> Self: ...
    def contains_only(self, *items: object) -> Self: ...
    def contains_sequence(self, *items: object) -> Self: ...
    def contains_duplicates(self) -> Self: ...
    def does_not_contain_duplicates(self) -> Self: ...
    def is_empty(self) -> Self: ...
    def is_not_empty(self) -> Self: ...
    def is_in(self, *items: _V) -> Self: ...
    def is_not_in(self, *items: _V) -> Self: ...



from _typeshed import Incomplete

from authlib.oauth2 import AuthorizationServer as _AuthorizationServer
from authlib.oauth2.rfc6750 import BearerTokenGenerator

from .requests import FlaskJsonRequest, FlaskOAuth2Request

class AuthorizationServer(_AuthorizationServer):
    def __init__(self, app=None, query_client=None, save_token=None) -> None: ...
    def init_app(self, app, query_client=None, save_token=None) -> None: ...
    scopes_supported: Incomplete
    def load_config(self, config) -> None: ...
    def query_client(self, client_id): ...
    def save_token(self, token, request): ...
    def get_error_uri(self, request, error): ...
    def create_oauth2_request(self, request) -> FlaskOAuth2Request: ...
    def create_json_request(self, request) -> FlaskJsonRequest: ...
    def handle_response(self, status_code, payload, headers): ...
    def send_signal(self, name, *args, **kwargs) -> None: ...
    def create_bearer_token_generator(self, config) -> BearerTokenGenerator: ...

def create_token_expires_in_generator(expires_in_conf=None): ...
def create_token_generator(token_generator_conf, length: int = 42): ...



from _typeshed import Incomplete

from ..rest import RestClientOptions
from ..types import TimeoutType

class CustomDomains:
    domain: Incomplete
    protocol: Incomplete
    client: Incomplete
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def all(self) -> list[dict[str, Incomplete]]: ...
    async def all_async(self) -> list[dict[str, Incomplete]]: ...
    def get(self, id: str) -> dict[str, Incomplete]: ...
    async def get_async(self, id: str) -> dict[str, Incomplete]: ...
    def delete(self, id: str): ...
    async def delete_async(self, id: str): ...
    def create_new(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def create_new_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def verify(self, id: str) -> dict[str, Incomplete]: ...
    async def verify_async(self, id: str) -> dict[str, Incomplete]: ...



from _typeshed import Incomplete

from authlib.oauth1 import ResourceProtector as _ResourceProtector

class ResourceProtector(_ResourceProtector):
    client_model: Incomplete
    token_model: Incomplete
    SUPPORTED_SIGNATURE_METHODS: Incomplete
    def __init__(self, client_model, token_model) -> None: ...
    def get_client_by_id(self, client_id): ...
    def get_token_credential(self, request): ...
    def exists_nonce(self, nonce, request) -> bool: ...
    def acquire_credential(self, request): ...
    def __call__(self, realm=None): ...



import sys
from _typeshed import SupportsWrite
from collections import deque
from typing import IO

__all__ = ["pprint", "pformat", "isreadable", "isrecursive", "saferepr", "PrettyPrinter", "pp"]

if sys.version_info >= (3, 15):
    # The `expand` parameter was added in Python 3.15.
    def pformat(
        object: object,
        indent: int = 1,
        width: int = 80,
        depth: int | None = None,
        *,
        compact: bool = False,
        expand: bool = False,
        sort_dicts: bool = True,
        underscore_numbers: bool = False,
    ) -> str: ...

else:
    def pformat(
        object: object,
        indent: int = 1,
        width: int = 80,
        depth: int | None = None,
        *,
        compact: bool = False,
        sort_dicts: bool = True,
        underscore_numbers: bool = False,
    ) -> str: ...

if sys.version_info >= (3, 15):
    # The `expand` parameter was added in Python 3.15.
    def pp(
        object: object,
        stream: IO[str] | None = None,
        indent: int = 1,
        width: int = 80,
        depth: int | None = None,
        *,
        compact: bool = False,
        expand: bool = False,
        sort_dicts: bool = False,
        underscore_numbers: bool = False,
    ) -> None: ...

else:
    def pp(
        object: object,
        stream: IO[str] | None = None,
        indent: int = 1,
        width: int = 80,
        depth: int | None = None,
        *,
        compact: bool = False,
        sort_dicts: bool = False,
        underscore_numbers: bool = False,
    ) -> None: ...

if sys.version_info >= (3, 15):
    # The `expand` parameter was added in Python 3.15.
    def pprint(
        object: object,
        stream: IO[str] | None = None,
        indent: int = 1,
        width: int = 80,
        depth: int | None = None,
        *,
        compact: bool = False,
        expand: bool = False,
        sort_dicts: bool = True,
        underscore_numbers: bool = False,
    ) -> None: ...

else:
    def pprint(
        object: object,
        stream: IO[str] | None = None,
        indent: int = 1,
        width: int = 80,
        depth: int | None = None,
        *,
        compact: bool = False,
        sort_dicts: bool = True,
        underscore_numbers: bool = False,
    ) -> None: ...

def isreadable(object: object) -> bool: ...
def isrecursive(object: object) -> bool: ...
def saferepr(object: object) -> str: ...

class PrettyPrinter:
    if sys.version_info >= (3, 15):
        # The `expand` parameter was added in Python 3.15.
        def __init__(
            self,
            indent: int = 1,
            width: int = 80,
            depth: int | None = None,
            stream: IO[str] | None = None,
            *,
            compact: bool = False,
            expand: bool = False,
            sort_dicts: bool = True,
            underscore_numbers: bool = False,
        ) -> None: ...
    else:
        def __init__(
            self,
            indent: int = 1,
            width: int = 80,
            depth: int | None = None,
            stream: IO[str] | None = None,
            *,
            compact: bool = False,
            sort_dicts: bool = True,
            underscore_numbers: bool = False,
        ) -> None: ...

    def pformat(self, object: object) -> str: ...
    def pprint(self, object: object) -> None: ...
    def isreadable(self, object: object) -> bool: ...
    def isrecursive(self, object: object) -> bool: ...
    def format(self, object: object, context: dict[int, int], maxlevels: int, level: int) -> tuple[str, bool, bool]: ...
    def _format(
        self, object: object, stream: SupportsWrite[str], indent: int, allowance: int, context: dict[int, int], level: int
    ) -> None: ...
    def _pprint_dict(
        self,
        object: dict[object, object],
        stream: SupportsWrite[str],
        indent: int,
        allowance: int,
        context: dict[int, int],
        level: int,
    ) -> None: ...
    def _pprint_list(
        self, object: list[object], stream: SupportsWrite[str], indent: int, allowance: int, context: dict[int, int], level: int
    ) -> None: ...
    def _pprint_tuple(
        self,
        object: tuple[object, ...],
        stream: SupportsWrite[str],
        indent: int,
        allowance: int,
        context: dict[int, int],
        level: int,
    ) -> None: ...
    def _pprint_set(
        self, object: set[object], stream: SupportsWrite[str], indent: int, allowance: int, context: dict[int, int], level: int
    ) -> None: ...
    def _pprint_deque(
        self, object: deque[object], stream: SupportsWrite[str], indent: int, allowance: int, context: dict[int, int], level: int
    ) -> None: ...
    def _format_dict_items(
        self,
        items: list[tuple[object, object]],
        stream: SupportsWrite[str],
        indent: int,
        allowance: int,
        context: dict[int, int],
        level: int,
    ) -> None: ...
    def _format_items(
        self, items: list[object], stream: SupportsWrite[str], indent: int, allowance: int, context: dict[int, int], level: int
    ) -> None: ...
    def _repr(self, object: object, context: dict[int, int], level: int) -> str: ...
    def _safe_repr(self, object: object, context: dict[int, int], maxlevels: int, level: int) -> tuple[str, bool, bool]: ...



from _typeshed import Incomplete

class KafkaError(RuntimeError):
    retriable: bool
    invalid_metadata: bool
    def __eq__(self, other): ...

class Cancelled(KafkaError):
    retriable: bool

class CommitFailedError(KafkaError):
    def __init__(self, *args) -> None: ...

class IllegalArgumentError(KafkaError): ...
class IllegalStateError(KafkaError): ...
class IncompatibleBrokerVersion(KafkaError): ...
class KafkaConfigurationError(KafkaError): ...

class KafkaConnectionError(KafkaError):
    retriable: bool
    invalid_metadata: bool

class KafkaProtocolError(KafkaError):
    retriable: bool

class CorrelationIdError(KafkaProtocolError):
    retriable: bool

class InvalidReceiveError(KafkaProtocolError): ...

class KafkaTimeoutError(KafkaError):
    retriable: bool

class MetadataEmptyBrokerList(KafkaError):
    retriable: bool

class NoBrokersAvailable(KafkaError):
    retriable: bool
    invalid_metadata: bool

class NoOffsetForPartitionError(KafkaError): ...

class NodeNotReadyError(KafkaError):
    retriable: bool

class QuotaViolationError(KafkaError): ...

class StaleMetadata(KafkaError):
    retriable: bool
    invalid_metadata: bool

class TooManyInFlightRequests(KafkaError):
    retriable: bool

class UnrecognizedBrokerVersion(KafkaError): ...
class UnsupportedCodecError(KafkaError): ...
class TransactionAbortedError(KafkaError): ...

class BrokerResponseError(KafkaError):
    errno: Incomplete
    message: Incomplete
    description: Incomplete

class AuthorizationError(BrokerResponseError): ...

class NoError(BrokerResponseError):
    errno: int
    message: str
    description: str

class UnknownError(BrokerResponseError):
    errno: int
    message: str
    description: str

class OffsetOutOfRangeError(BrokerResponseError):
    errno: int
    message: str
    description: str

class CorruptRecordError(BrokerResponseError):
    errno: int
    message: str
    description: str

CorruptRecordException = CorruptRecordError

class UnknownTopicOrPartitionError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool
    invalid_metadata: bool

class InvalidFetchRequestError(BrokerResponseError):
    errno: int
    message: str
    description: str

class LeaderNotAvailableError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool
    invalid_metadata: bool

class NotLeaderForPartitionError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool
    invalid_metadata: bool

class RequestTimedOutError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class BrokerNotAvailableError(BrokerResponseError):
    errno: int
    message: str
    description: str

class ReplicaNotAvailableError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool
    invalid_metadata: bool

class MessageSizeTooLargeError(BrokerResponseError):
    errno: int
    message: str
    description: str

class StaleControllerEpochError(BrokerResponseError):
    errno: int
    message: str
    description: str

class OffsetMetadataTooLargeError(BrokerResponseError):
    errno: int
    message: str
    description: str

class NetworkExceptionError(BrokerResponseError):
    errno: int
    message: str
    retriable: bool
    invalid_metadata: bool

class CoordinatorLoadInProgressError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class CoordinatorNotAvailableError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class NotCoordinatorError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidTopicError(BrokerResponseError):
    errno: int
    message: str
    description: str

class RecordListTooLargeError(BrokerResponseError):
    errno: int
    message: str
    description: str

class NotEnoughReplicasError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class NotEnoughReplicasAfterAppendError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidRequiredAcksError(BrokerResponseError):
    errno: int
    message: str
    description: str

class IllegalGenerationError(BrokerResponseError):
    errno: int
    message: str
    description: str

class InconsistentGroupProtocolError(BrokerResponseError):
    errno: int
    message: str
    description: str

class InvalidGroupIdError(BrokerResponseError):
    errno: int
    message: str
    description: str

class UnknownMemberIdError(BrokerResponseError):
    errno: int
    message: str
    description: str

class InvalidSessionTimeoutError(BrokerResponseError):
    errno: int
    message: str
    description: str

class RebalanceInProgressError(BrokerResponseError):
    errno: int
    message: str
    description: str

class InvalidCommitOffsetSizeError(BrokerResponseError):
    errno: int
    message: str
    description: str

class TopicAuthorizationFailedError(AuthorizationError):
    errno: int
    message: str
    description: str

class GroupAuthorizationFailedError(AuthorizationError):
    errno: int
    message: str
    description: str

class ClusterAuthorizationFailedError(AuthorizationError):
    errno: int
    message: str
    description: str

class InvalidTimestampError(BrokerResponseError):
    errno: int
    message: str
    description: str

class UnsupportedSaslMechanismError(BrokerResponseError):
    errno: int
    message: str
    description: str

class IllegalSaslStateError(BrokerResponseError):
    errno: int
    message: str
    description: str

class UnsupportedVersionError(BrokerResponseError):
    errno: int
    message: str
    description: str

class TopicAlreadyExistsError(BrokerResponseError):
    errno: int
    message: str
    description: str

class InvalidPartitionsError(BrokerResponseError):
    errno: int
    message: str
    description: str

class InvalidReplicationFactorError(BrokerResponseError):
    errno: int
    message: str
    description: str

class InvalidReplicationAssignmentError(BrokerResponseError):
    errno: int
    message: str
    description: str

class InvalidConfigurationError(BrokerResponseError):
    errno: int
    message: str
    description: str

class NotControllerError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidRequestError(BrokerResponseError):
    errno: int
    message: str
    description: str

class UnsupportedForMessageFormatError(BrokerResponseError):
    errno: int
    message: str
    description: str

class PolicyViolationError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class OutOfOrderSequenceNumberError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class DuplicateSequenceNumberError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidProducerEpochError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidTxnStateError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidProducerIdMappingError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidTransactionTimeoutError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class ConcurrentTransactionsError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class TransactionCoordinatorFencedError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class TransactionalIdAuthorizationFailedError(AuthorizationError):
    errno: int
    message: str
    description: str
    retriable: bool

class SecurityDisabledError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class OperationNotAttemptedError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class KafkaStorageError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool
    invalid_metadata: bool

class LogDirNotFoundError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class SaslAuthenticationFailedError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class UnknownProducerIdError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class ReassignmentInProgressError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class DelegationTokenAuthDisabledError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class DelegationTokenNotFoundError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class DelegationTokenOwnerMismatchError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class DelegationTokenRequestNotAllowedError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class DelegationTokenAuthorizationFailedError(AuthorizationError):
    errno: int
    message: str
    description: str
    retriable: bool

class DelegationTokenExpiredError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidPrincipalTypeError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class NonEmptyGroupError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class GroupIdNotFoundError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class FetchSessionIdNotFoundError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidFetchSessionEpochError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class ListenerNotFoundError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool
    invalid_metadata: bool

class TopicDeletionDisabledError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class FencedLeaderEpochError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool
    invalid_metadata: bool

class UnknownLeaderEpochError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool
    invalid_metadata: bool

class UnsupportedCompressionTypeError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class StaleBrokerEpochError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class OffsetNotAvailableError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class MemberIdRequiredError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class PreferredLeaderNotAvailableError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool
    invalid_metadata: bool

class GroupMaxSizeReachedError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class FencedInstanceIdError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class EligibleLeadersNotAvailableError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool
    invalid_metadata: bool

class ElectionNotNeededError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool
    invalid_metadata: bool

class NoReassignmentInProgressError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class GroupSubscribedToTopicError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidRecordError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class UnstableOffsetCommitError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class ThrottlingQuotaExceededError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class ProducerFencedError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class ResourceNotFoundError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class DuplicateResourceError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class UnacceptableCredentialError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InconsistentVoterSetError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidUpdateVersionError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class FeatureUpdateFailedError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class PrincipalDeserializationFailureError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class SnapshotNotFoundError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class PositionOutOfRangeError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class UnknownTopicIdError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool
    invalid_metadata: bool

class DuplicateBrokerRegistrationError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class BrokerIdNotRegisteredError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InconsistentTopicIdError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool
    invalid_metadata: bool

class InconsistentClusterIdError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class TransactionalIdNotFoundError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class FetchSessionTopicIdError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class IneligibleReplicaError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class NewLeaderElectedError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class OffsetMovedToTieredStorageError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class FencedMemberEpochError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class UnreleasedInstanceIdError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class UnsupportedAssignorError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class StaleMemberEpochError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class MismatchedEndpointTypeError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class UnsupportedEndpointTypeError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class UnknownControllerIdError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class UnknownSubscriptionIdError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class TelemetryTooLargeError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidRegistrationError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class TransactionAbortableError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidRecordStateError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class ShareSessionNotFoundError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidShareSessionEpochError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class FencedStateEpochError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class InvalidVoterKeyError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class DuplicateVoterError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

class VoterNotFoundError(BrokerResponseError):
    errno: int
    message: str
    description: str
    retriable: bool

kafka_errors: Incomplete

def for_code(error_code): ...



from collections.abc import Sequence

class NewPartitions:
    total_count: int
    new_assignments: Sequence[Sequence[int]] | None
    def __init__(self, total_count: int, new_assignments: Sequence[Sequence[int]] | None = None) -> None: ...



from collections.abc import Callable, Container, Hashable, Iterable
from io import TextIOWrapper
from re import Pattern
from typing import Any, Final, Protocol, TypeVar, type_check_only

_T = TypeVar("_T")
_H = TypeVar("_H", bound=Hashable)

split_path_re: Final[Pattern[str]]
doctype_lookup_re: Final[Pattern[str]]
tag_re: Final[Pattern[str]]
xml_decl_re: Final[Pattern[str]]

class ClassNotFound(ValueError): ...
class OptionError(Exception): ...

@type_check_only
class _SupportsGetStrWithDefault(Protocol):
    def get(self, item: str, default: Any, /) -> Any: ...

# 'options' contains the **kwargs of an arbitrary function.
def get_choice_opt(
    options: _SupportsGetStrWithDefault, optname: str, allowed: Container[_T], default: _T | None = None, normcase: bool = False
) -> _T: ...
def get_bool_opt(options: _SupportsGetStrWithDefault, optname: str, default: bool | None = None) -> bool: ...
def get_int_opt(options: _SupportsGetStrWithDefault, optname: str, default: int | None = None) -> int: ...

# Return type and type of 'default' depend on the signature of the function whose **kwargs
# are being processed.
def get_list_opt(
    options: _SupportsGetStrWithDefault, optname: str, default: list[Any] | tuple[Any, ...] | None = None
) -> list[Any]: ...
def docstring_headline(obj: object) -> str: ...
def make_analysator(f: Callable[[str], float]) -> Callable[[str], float]: ...
def shebang_matches(text: str, regex: str) -> bool: ...
def doctype_matches(text: str, regex: str) -> bool: ...
def html_doctype_matches(text: str) -> bool: ...
def looks_like_xml(text: str) -> bool: ...
def surrogatepair(c: int) -> int: ...
def format_lines(var_name: str, seq: Iterable[str], raw: bool = False, indent_level: int = 0) -> str: ...
def duplicates_removed(it: Iterable[_H], already_seen: Container[_H] = ()) -> list[_H]: ...

class Future:
    def get(self) -> None: ...

def guess_decode(text: bytes) -> tuple[str, str]: ...

# If 'term' has an 'encoding' attribute, it should be a str. Otherwise any object is accepted.
def guess_decode_from_terminal(text: bytes, term: Any) -> tuple[str, str]: ...
def terminal_encoding(term: Any) -> str: ...

class UnclosingTextIOWrapper(TextIOWrapper):
    def close(self) -> None: ...



from typing import Any

class GeneratedProtocolMessageType(type):
    def __new__(cls, name: str, bases: tuple[type, ...], dictionary: dict[str, Any]) -> GeneratedProtocolMessageType: ...
    def __init__(cls, name: str, bases: tuple[type, ...], dictionary: dict[str, Any]) -> None: ...



from _typeshed import Incomplete

from kafka.protocol.api import Request, Response

class AddOffsetsToTxnResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class AddOffsetsToTxnResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class AddOffsetsToTxnResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class AddOffsetsToTxnRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = AddOffsetsToTxnResponse_v0
    SCHEMA: Incomplete

class AddOffsetsToTxnRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = AddOffsetsToTxnResponse_v1
    SCHEMA: Incomplete

class AddOffsetsToTxnRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = AddOffsetsToTxnResponse_v2
    SCHEMA: Incomplete

AddOffsetsToTxnRequest: Incomplete
AddOffsetsToTxnResponse: Incomplete



from pika.adapters.asyncio_connection import AsyncioConnection as AsyncioConnection
from pika.adapters.base_connection import BaseConnection as BaseConnection
from pika.adapters.blocking_connection import BlockingConnection as BlockingConnection
from pika.adapters.select_connection import IOLoop as IOLoop, SelectConnection as SelectConnection

__all__ = ["AsyncioConnection", "BaseConnection", "BlockingConnection", "SelectConnection", "IOLoop"]



from _typeshed import Incomplete
from typing import Final

__version__: Final[str]

def bezierArc(
    x1, y1, x2, y2, startAng: int = 0, extent: int = 90
) -> list[tuple[Incomplete, Incomplete, Incomplete, Incomplete, Incomplete, Incomplete, Incomplete, Incomplete]]: ...



from _typeshed import Incomplete

class ReverseDnsSetting:
    OFF: Incomplete
    REQUIRE_RESOLVE_ALL_ADDRESSES: Incomplete
    REQUIRE_RESOLVE_IP_ADDRESSES_ONLY: Incomplete
    OPTIONAL_RESOLVE_ALL_ADDRESSES: Incomplete
    OPTIONAL_RESOLVE_IP_ADDRESSES_ONLY: Incomplete
    SUPPORTED_VALUES: Incomplete

def get_hostname_by_addr(addr, success_required: bool = True): ...
def is_ip_addr(addr): ...



from _typeshed import Incomplete, SupportsWrite
from collections.abc import Iterable
from typing import TypeVar

from pygments.formatter import Formatter
from pygments.token import _TokenType

_T = TypeVar("_T", str, bytes)

__all__ = ["IRCFormatter"]

class IRCFormatter(Formatter[_T]):
    darkbg: Incomplete
    colorscheme: Incomplete
    linenos: Incomplete
    def format_unencoded(self, tokensource: Iterable[tuple[_TokenType, str]], outfile: SupportsWrite[str]) -> None: ...



from _typeshed import SupportsItems
from collections.abc import Iterator, KeysView
from enum import Enum
from http.cookiejar import Cookie, CookiePolicy, MozillaCookieJar
from http.cookies import Morsel, SimpleCookie
from typing import Any, Final, TextIO, TypeVar

from . import _LoggerProtocol
from .minicurses import MultilinePrinter
from .utils._utils import YoutubeDLError
from .YoutubeDL import YoutubeDL

CHROMIUM_BASED_BROWSERS: Final[set[str]]
SUPPORTED_BROWSERS: Final[set[str]]

class _LinuxKeyring(Enum):
    BASICTEXT = 5
    GNOMEKEYRING = 4
    KWALLET = 1
    KWALLET5 = 2
    KWALLET6 = 3

SUPPORTED_KEYRINGS: Final[KeysView[str]]

class YDLLogger(_LoggerProtocol):
    def warning(self, message: str, only_once: bool = False) -> None: ...  # type: ignore[override]

    class ProgressBar(MultilinePrinter):
        def print(self, message: str) -> None: ...

    def progress_bar(self) -> ProgressBar: ...

class CookieLoadError(YoutubeDLError): ...

class YoutubeDLCookieJar(MozillaCookieJar):
    def __init__(self, filename: str | None = None, delayload: bool = False, policy: CookiePolicy | None = None) -> None: ...
    def open(self, file: str, *, write: bool = False) -> Iterator[TextIO]: ...
    def get_cookie_header(self, url: str) -> str: ...
    def get_cookies_for_url(self, url: str) -> list[Cookie]: ...
    def load(self, filename: str | None = None, ignore_discard: bool = True, ignore_expires: bool = True) -> None: ...
    def save(self, filename: str | None = None, ignore_discard: bool = True, ignore_expires: bool = True) -> None: ...

def load_cookies(cookie_file: str, browser_specification: str | None, ydl: YoutubeDL) -> YoutubeDLCookieJar: ...
def extract_cookies_from_browser(
    browser_name: str,
    profile: str | None = None,
    logger: _LoggerProtocol = ...,
    *,
    keyring: _LinuxKeyring | None = None,
    container: str | None = None,
) -> YoutubeDLCookieJar: ...

_T = TypeVar("_T", bound=MozillaCookieJar)

def parse_safari_cookies(data: bytes, jar: _T | None = None, logger: _LoggerProtocol = ...) -> _T: ...

class ChromeCookieDecryptor:
    def decrypt(self, encrypted_value: bytes) -> str: ...

class LinuxChromeCookieDecryptor(ChromeCookieDecryptor):
    def __init__(
        self,
        browser_keyring_name: str,
        logger: _LoggerProtocol,
        *,
        keyring: _LinuxKeyring | None = None,
        meta_version: int | None = None,
    ) -> None: ...
    @staticmethod
    def derive_key(password: bytes) -> bytes: ...

class MacChromeCookieDecryptor(ChromeCookieDecryptor):
    def __init__(self, browser_keyring_name: str, logger: YDLLogger, meta_version: int | None = None) -> None: ...
    @staticmethod
    def derive_key(password: bytes) -> bytes: ...

class WindowsChromeCookieDecryptor(ChromeCookieDecryptor):
    def __init__(self, browser_root: str, logger: YDLLogger, meta_version: int | None = None) -> None: ...

def get_cookie_decryptor(
    browser_root: str,
    browser_keyring_name: str,
    logger: _LoggerProtocol,
    *,
    keyring: _LinuxKeyring | None = None,
    meta_version: int | None = None,
) -> ChromeCookieDecryptor: ...

class ParserError(Exception): ...

class DataParser:
    def __init__(self, data: bytes, logger: YDLLogger) -> None: ...
    def read_bytes(self, num_bytes: int) -> bytes: ...
    def expect_bytes(self, expected_value: bytes, message: str) -> None: ...
    def read_uint(self, big_endian: bool = False) -> int: ...
    def read_double(self, big_endian: bool = False) -> float: ...
    def read_cstring(self) -> bytes: ...
    def skip(self, num_bytes: int, description: str = "unknown") -> None: ...
    def skip_to(self, offset: int, description: str = "unknown") -> None: ...
    def skip_to_end(self, description: str = "unknown") -> None: ...

def pbkdf2_sha1(password: bytes, salt: bytes, iterations: int, key_length: int) -> bytes: ...

class LenientSimpleCookie(SimpleCookie):
    def load(self, data: str | SupportsItems[str, str | Morsel[Any]]) -> None: ...



from typing import Any, ClassVar, Final, NamedTuple, type_check_only
from typing_extensions import Self

from docutils.transforms import Transform

__docformat__: Final = "reStructuredText"
__version__: Final[str]

@type_check_only
class _VersionInfo(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: str
    serial: int
    release: bool

class VersionInfo(_VersionInfo):
    __slots__ = ()
    def __new__(
        cls, major: int = 0, minor: int = 0, micro: int = 0, releaselevel: str = "final", serial: int = 0, release: bool = True
    ) -> Self: ...

__version_info__: Final[VersionInfo]
__version_details__: Final[str]

class ApplicationError(Exception): ...
class DataError(ApplicationError): ...

class SettingsSpec:
    settings_spec: ClassVar[tuple[Any, ...]]  # Mixed tuple structure; uses Any for flexibility in nested option definitions
    settings_defaults: ClassVar[dict[Any, Any] | None]
    settings_default_overrides: ClassVar[dict[Any, Any] | None]
    relative_path_settings: ClassVar[tuple[Any, ...]]
    config_section: ClassVar[str | None]
    config_section_dependencies: ClassVar[tuple[str, ...] | None]

class TransformSpec:
    def get_transforms(self) -> list[type[Transform]]: ...
    default_transforms: ClassVar[tuple[Any, ...]]
    unknown_reference_resolvers: ClassVar[list[Any]]

class Component(SettingsSpec, TransformSpec):
    component_type: ClassVar[str | None]
    supported: ClassVar[tuple[str, ...]]
    def supports(self, format: str) -> bool: ...



from collections.abc import Iterator

from dateparser.conf import Settings
from dateparser.languages.locale import Locale

class BaseLanguageDetector:
    languages: list[Locale]
    def __init__(self, languages: list[Locale]) -> None: ...
    def iterate_applicable_languages(
        self, date_string: str, modify: bool = False, settings: Settings | None = None
    ) -> Iterator[Locale]: ...

class AutoDetectLanguage(BaseLanguageDetector):
    language_pool: list[Locale]
    allow_redetection: bool
    def __init__(self, languages: list[Locale], allow_redetection: bool = False) -> None: ...
    def iterate_applicable_languages(
        self, date_string: str, modify: bool = False, settings: Settings | None = None
    ) -> Iterator[Locale]: ...

class ExactLanguages(BaseLanguageDetector):
    def __init__(self, languages: list[Locale]) -> None: ...
    def iterate_applicable_languages(
        self, date_string: str, modify: bool = False, settings: Settings | None = None
    ) -> Iterator[Locale]: ...



from kafka.metrics.stats.avg import Avg as Avg
from kafka.metrics.stats.count import Count as Count
from kafka.metrics.stats.histogram import Histogram as Histogram
from kafka.metrics.stats.max_stat import Max as Max
from kafka.metrics.stats.min_stat import Min as Min
from kafka.metrics.stats.percentile import Percentile as Percentile
from kafka.metrics.stats.percentiles import Percentiles as Percentiles
from kafka.metrics.stats.rate import Rate as Rate
from kafka.metrics.stats.sensor import Sensor as Sensor
from kafka.metrics.stats.total import Total as Total

__all__ = ["Avg", "Count", "Histogram", "Max", "Min", "Percentile", "Percentiles", "Rate", "Sensor", "Total"]



from _typeshed import Incomplete
from typing import Final

from authlib.jose.rfc7516 import JWEEncAlgorithm

class CBCHS2EncAlgorithm(JWEEncAlgorithm):
    IV_SIZE: int
    name: str
    description: str
    key_size: int
    key_len: int
    CEK_SIZE: int
    hash_alg: Incomplete
    def __init__(self, key_size: int, hash_type: int | str) -> None: ...
    def encrypt(self, msg, aad, iv, key) -> tuple[bytes, bytes]: ...
    def decrypt(self, ciphertext, aad, iv, tag, key) -> bytes: ...

class GCMEncAlgorithm(JWEEncAlgorithm):
    IV_SIZE: int
    name: str
    description: str
    key_size: int
    CEK_SIZE: int
    def __init__(self, key_size: int) -> None: ...
    def encrypt(self, msg, aad, iv, key) -> tuple[bytes, bytes]: ...
    def decrypt(self, ciphertext, aad, iv, tag, key) -> bytes: ...

JWE_ENC_ALGORITHMS: Final[list[CBCHS2EncAlgorithm | GCMEncAlgorithm]]



from kafka.metrics.compound_stat import NamedMeasurable as NamedMeasurable
from kafka.metrics.dict_reporter import DictReporter as DictReporter
from kafka.metrics.kafka_metric import KafkaMetric as KafkaMetric
from kafka.metrics.measurable import AnonMeasurable as AnonMeasurable
from kafka.metrics.metric_config import MetricConfig as MetricConfig
from kafka.metrics.metric_name import MetricName as MetricName
from kafka.metrics.metrics import Metrics as Metrics
from kafka.metrics.quota import Quota as Quota

__all__ = ["AnonMeasurable", "DictReporter", "KafkaMetric", "MetricConfig", "MetricName", "Metrics", "NamedMeasurable", "Quota"]



from _typeshed import Incomplete

log: Incomplete

class ClusterMetadata:
    DEFAULT_CONFIG: Incomplete
    need_all_topic_metadata: bool
    unauthorized_topics: Incomplete
    internal_topics: Incomplete
    controller: Incomplete
    cluster_id: Incomplete
    config: Incomplete
    def __init__(self, **configs) -> None: ...
    def is_bootstrap(self, node_id): ...
    def brokers(self): ...
    def broker_metadata(self, broker_id): ...
    def partitions_for_topic(self, topic): ...
    def available_partitions_for_topic(self, topic): ...
    def leader_for_partition(self, partition): ...
    def leader_epoch_for_partition(self, partition): ...
    def partitions_for_broker(self, broker_id): ...
    def coordinator_for_group(self, group): ...
    def ttl(self): ...
    def refresh_backoff(self): ...
    def request_update(self): ...
    @property
    def need_update(self): ...
    def topics(self, exclude_internal_topics: bool = True): ...
    def failed_update(self, exception) -> None: ...
    def update_metadata(self, metadata): ...
    def add_listener(self, listener) -> None: ...
    def remove_listener(self, listener) -> None: ...
    def add_coordinator(self, response, coord_type, coord_key): ...
    def with_partitions(self, partitions_to_add): ...

def collect_hosts(hosts, randomize: bool = True): ...



from _typeshed import SupportsGetItem, SupportsLenAndGetItem, SupportsRichComparisonT
from collections.abc import Callable, MutableSequence
from typing import TypeVar, overload

_T = TypeVar("_T")

@overload
def bisect_left(
    a: SupportsLenAndGetItem[SupportsRichComparisonT],
    x: SupportsRichComparisonT,
    lo: int = 0,
    hi: int | None = None,
    *,
    key: None = None,
) -> int: ...
@overload
def bisect_left(
    a: SupportsGetItem[int, SupportsRichComparisonT], x: SupportsRichComparisonT, lo: int, hi: int, *, key: None = None
) -> int: ...
@overload
def bisect_left(
    a: SupportsGetItem[int, SupportsRichComparisonT], x: SupportsRichComparisonT, lo: int = 0, *, hi: int, key: None = None
) -> int: ...
@overload
def bisect_left(
    a: SupportsLenAndGetItem[_T],
    x: SupportsRichComparisonT,
    lo: int = 0,
    hi: int | None = None,
    *,
    key: Callable[[_T], SupportsRichComparisonT],
) -> int: ...
@overload
def bisect_left(
    a: SupportsGetItem[int, _T], x: SupportsRichComparisonT, lo: int, hi: int, *, key: Callable[[_T], SupportsRichComparisonT]
) -> int: ...
@overload
def bisect_left(
    a: SupportsGetItem[int, _T], x: SupportsRichComparisonT, lo: int = 0, *, hi: int, key: Callable[[_T], SupportsRichComparisonT]
) -> int: ...

@overload
def bisect_right(
    a: SupportsLenAndGetItem[SupportsRichComparisonT],
    x: SupportsRichComparisonT,
    lo: int = 0,
    hi: int | None = None,
    *,
    key: None = None,
) -> int: ...
@overload
def bisect_right(
    a: SupportsGetItem[int, SupportsRichComparisonT], x: SupportsRichComparisonT, lo: int, hi: int, *, key: None = None
) -> int: ...
@overload
def bisect_right(
    a: SupportsGetItem[int, SupportsRichComparisonT], x: SupportsRichComparisonT, lo: int = 0, *, hi: int, key: None = None
) -> int: ...
@overload
def bisect_right(
    a: SupportsLenAndGetItem[_T],
    x: SupportsRichComparisonT,
    lo: int = 0,
    hi: int | None = None,
    *,
    key: Callable[[_T], SupportsRichComparisonT],
) -> int: ...
@overload
def bisect_right(
    a: SupportsGetItem[int, _T], x: SupportsRichComparisonT, lo: int, hi: int, *, key: Callable[[_T], SupportsRichComparisonT]
) -> int: ...
@overload
def bisect_right(
    a: SupportsGetItem[int, _T], x: SupportsRichComparisonT, lo: int = 0, *, hi: int, key: Callable[[_T], SupportsRichComparisonT]
) -> int: ...

@overload
def insort_left(
    a: MutableSequence[SupportsRichComparisonT],
    x: SupportsRichComparisonT,
    lo: int = 0,
    hi: int | None = None,
    *,
    key: None = None,
) -> None: ...
@overload
def insort_left(
    a: MutableSequence[_T], x: _T, lo: int = 0, hi: int | None = None, *, key: Callable[[_T], SupportsRichComparisonT]
) -> None: ...

@overload
def insort_right(
    a: MutableSequence[SupportsRichComparisonT],
    x: SupportsRichComparisonT,
    lo: int = 0,
    hi: int | None = None,
    *,
    key: None = None,
) -> None: ...
@overload
def insort_right(
    a: MutableSequence[_T], x: _T, lo: int = 0, hi: int | None = None, *, key: Callable[[_T], SupportsRichComparisonT]
) -> None: ...



from behave.fixture import fixture as fixture, use_fixture as use_fixture
from behave.step_registry import (
    Given as Given,
    Step as Step,
    Then as Then,
    When as When,
    given as given,
    step as step,
    then as then,
    when as when,
)

__all__ = ["given", "when", "then", "step", "Given", "When", "Then", "Step", "use_fixture", "fixture"]



from typing import Any

from urllib3.exceptions import HTTPError as BaseHTTPError

from .compat import JSONDecodeError as CompatJSONDecodeError
from .models import Request, Response
from .sessions import PreparedRequest

class RequestException(OSError):
    response: Response | None
    request: Request | PreparedRequest | None
    def __init__(
        self, *args: object, request: Request | PreparedRequest | None = ..., response: Response | None = ...
    ) -> None: ...

class InvalidJSONError(RequestException): ...
class JSONDecodeError(InvalidJSONError, CompatJSONDecodeError): ...

class HTTPError(RequestException):
    request: Request | PreparedRequest | Any
    response: Response | Any

class ConnectionError(RequestException): ...
class ProxyError(ConnectionError): ...
class SSLError(ConnectionError): ...
class Timeout(RequestException): ...
class ConnectTimeout(ConnectionError, Timeout): ...
class ReadTimeout(Timeout): ...
class URLRequired(RequestException): ...
class TooManyRedirects(RequestException): ...
class MissingSchema(RequestException, ValueError): ...
class InvalidSchema(RequestException, ValueError): ...
class InvalidURL(RequestException, ValueError): ...
class InvalidHeader(RequestException, ValueError): ...
class InvalidProxyURL(InvalidURL): ...
class ChunkedEncodingError(RequestException): ...
class ContentDecodingError(RequestException, BaseHTTPError): ...
class StreamConsumedError(RequestException, TypeError): ...
class RetryError(RequestException): ...
class UnrewindableBodyError(RequestException): ...
class RequestsWarning(Warning): ...
class FileModeWarning(RequestsWarning, DeprecationWarning): ...
class RequestsDependencyWarning(RequestsWarning): ...



from authlib.oauth1 import (
    SIGNATURE_HMAC_SHA1 as SIGNATURE_HMAC_SHA1,
    SIGNATURE_PLAINTEXT as SIGNATURE_PLAINTEXT,
    SIGNATURE_RSA_SHA1 as SIGNATURE_RSA_SHA1,
    SIGNATURE_TYPE_BODY as SIGNATURE_TYPE_BODY,
    SIGNATURE_TYPE_HEADER as SIGNATURE_TYPE_HEADER,
    SIGNATURE_TYPE_QUERY as SIGNATURE_TYPE_QUERY,
)

from ..base_client import OAuthError as OAuthError
from .assertion_client import AssertionClient as AssertionClient, AsyncAssertionClient as AsyncAssertionClient
from .oauth1_client import AsyncOAuth1Client as AsyncOAuth1Client, OAuth1Auth as OAuth1Auth, OAuth1Client as OAuth1Client
from .oauth2_client import (
    AsyncOAuth2Client as AsyncOAuth2Client,
    OAuth2Auth as OAuth2Auth,
    OAuth2Client as OAuth2Client,
    OAuth2ClientAuth as OAuth2ClientAuth,
)

__all__ = [
    "OAuthError",
    "OAuth1Auth",
    "AsyncOAuth1Client",
    "OAuth1Client",
    "SIGNATURE_HMAC_SHA1",
    "SIGNATURE_RSA_SHA1",
    "SIGNATURE_PLAINTEXT",
    "SIGNATURE_TYPE_HEADER",
    "SIGNATURE_TYPE_QUERY",
    "SIGNATURE_TYPE_BODY",
    "OAuth2Auth",
    "OAuth2ClientAuth",
    "OAuth2Client",
    "AsyncOAuth2Client",
    "AssertionClient",
    "AsyncAssertionClient",
]



from _typeshed import Incomplete

from kafka.metrics.compound_stat import AbstractCompoundStat
from kafka.metrics.stats.sampled_stat import AbstractSampledStat

class BucketSizing:
    CONSTANT: int
    LINEAR: int

class Percentiles(AbstractSampledStat, AbstractCompoundStat):
    bin_scheme: Incomplete
    def __init__(self, size_in_bytes, bucketing, max_val, min_val: float = 0.0, percentiles=None) -> None: ...
    def stats(self): ...
    def value(self, config, now, quantile): ...
    def combine(self, samples, config, now): ...
    def new_sample(self, time_ms): ...
    def update(self, sample, config, value, time_ms) -> None: ...

    class HistogramSample(AbstractSampledStat.Sample):
        histogram: Incomplete
        def __init__(self, scheme, now) -> None: ...



from win32.lib.pywintypes import error as error

def GetHandle() -> int: ...
def GetTracer(): ...
def InitRead() -> None: ...
def InitWrite() -> None: ...
def TermRead() -> None: ...
def TermWrite() -> None: ...
def blockingread(milliSeconds: int = ..., /) -> str: ...
def flush() -> None: ...
def read() -> str: ...
def setprint() -> None: ...
def write(data: str, /) -> None: ...



import abc

class AbstractMetricsReporter(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def init(self, metrics): ...
    @abc.abstractmethod
    def metric_change(self, metric): ...
    @abc.abstractmethod
    def metric_removal(self, metric): ...
    @abc.abstractmethod
    def configure(self, configs): ...
    @abc.abstractmethod
    def close(self): ...



import logging

from kafka.admin import KafkaAdminClient as KafkaAdminClient
from kafka.client_async import KafkaClient as KafkaClient
from kafka.conn import BrokerConnection as BrokerConnection
from kafka.consumer import KafkaConsumer as KafkaConsumer
from kafka.consumer.subscription_state import ConsumerRebalanceListener as ConsumerRebalanceListener
from kafka.producer import KafkaProducer as KafkaProducer

__all__ = ["BrokerConnection", "ConsumerRebalanceListener", "KafkaAdminClient", "KafkaClient", "KafkaConsumer", "KafkaProducer"]

class NullHandler(logging.Handler):
    def emit(self, record) -> None: ...



from _typeshed import ConvertibleToInt
from typing import Final, Literal
from typing_extensions import Self

__version__: Final[str]

def getStdMonthNames() -> list[str]: ...
def getStdShortMonthNames() -> list[str]: ...
def getStdDayNames() -> list[str]: ...
def getStdShortDayNames() -> list[str]: ...
def isLeapYear(year: int) -> Literal[0, 1]: ...

class NormalDateException(Exception): ...

class NormalDate:
    def __init__(self, normalDate=None) -> None: ...
    def add(self, days) -> None: ...
    def __add__(self, days: int) -> Self: ...
    def __radd__(self, days: int) -> Self: ...
    def clone(self) -> Self: ...
    def __lt__(self, other) -> bool: ...
    def __le__(self, other) -> bool: ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __ge__(self, other) -> bool: ...
    def __gt__(self, other) -> bool: ...
    def day(self) -> int: ...
    def dayOfWeek(self) -> int: ...
    @property
    def __day_of_week_name__(self): ...
    def dayOfWeekAbbrev(self): ...
    def dayOfWeekName(self): ...
    def dayOfYear(self) -> int: ...
    def daysBetweenDates(self, normalDate) -> int: ...
    def equals(self, target) -> bool | Literal[0]: ...
    def endOfMonth(self) -> Self: ...
    def firstDayOfMonth(self) -> Self: ...
    def formatUS(self) -> str: ...
    def formatUSCentury(self) -> str: ...
    def formatMS(self, fmt): ...
    def __hash__(self) -> int: ...
    def __int__(self) -> int: ...
    def isLeapYear(self) -> Literal[0, 1]: ...
    def lastDayOfMonth(self) -> int: ...
    def localeFormat(self) -> str: ...
    def month(self) -> int: ...
    @property
    def __month_name__(self): ...
    def monthAbbrev(self): ...
    def monthName(self): ...
    def normalize(self, scalar) -> None: ...
    def range(self, days) -> list[NormalDate]: ...
    def scalar(self) -> int: ...
    def setDay(self, day) -> None: ...
    def setMonth(self, month) -> None: ...
    normalDate: int | None
    def setNormalDate(self, normalDate) -> None: ...
    def setYear(self, year) -> None: ...
    def __sub__(self, v): ...
    def __rsub__(self, v): ...
    def toTuple(self) -> tuple[int, int, int]: ...
    def year(self) -> int: ...

def bigBang() -> NormalDate: ...
def bigCrunch() -> NormalDate: ...
def dayOfWeek(y: int, m: int, d: int) -> int: ...
def firstDayOfYear(year: int) -> int: ...
def FND(d): ...

Epoch: NormalDate
ND = NormalDate
BDEpoch: ND
BDEpochScalar: int

class BusinessDate(NormalDate):
    def add(self, days: int) -> None: ...
    def __add__(self, days: int) -> Self: ...
    def __sub__(self, v): ...
    def asNormalDate(self) -> ND: ...
    def daysBetweenDates(self, normalDate) -> int: ...
    def normalize(self, i: ConvertibleToInt) -> None: ...
    def scalar(self): ...
    def setNormalDate(self, normalDate) -> None: ...



import re
import threading
from _typeshed import Incomplete, SupportsKeysAndGetItem, Unused
from collections.abc import Callable, Generator, Iterable, Iterator
from datetime import datetime
from decimal import Decimal
from types import TracebackType
from typing import Any, ClassVar, Final, Literal, NamedTuple, NoReturn, TypeVar, overload, type_check_only
from typing_extensions import Self, TypeIs
from uuid import UUID

def callable_(c: object) -> TypeIs[Callable[..., object]]: ...

multi_types: tuple[type[Incomplete], ...]

def reraise(tp: Unused, value: BaseException, tb: TracebackType | None = None) -> NoReturn: ...

_T = TypeVar("_T")
_VT = TypeVar("_VT")
_F = TypeVar("_F", bound=Callable[..., Any])

class attrdict(dict[str, _VT]):
    def __getattr__(self, attr: str) -> _VT: ...
    def __setattr__(self, attr: str, value: _VT) -> None: ...
    # calls dict.update()
    def __iadd__(self, rhs: SupportsKeysAndGetItem[str, _VT] | Iterable[tuple[str, _VT]]) -> Self: ...
    def __add__(self, rhs: SupportsKeysAndGetItem[str, _VT] | Iterable[tuple[str, _VT]]) -> attrdict[_VT]: ...

OP: attrdict[str]
DJANGO_MAP: attrdict[Incomplete]
JOIN: attrdict[str]
ROW: attrdict[int]
PREFETCH_TYPE: attrdict[int]
SCOPE_NORMAL: Final = 1
SCOPE_SOURCE: Final = 2
SCOPE_VALUES: Final = 4
SCOPE_CTE: Final = 8
SCOPE_COLUMN: Final = 16
CSQ_PARENTHESES_NEVER: Final = 0
CSQ_PARENTHESES_ALWAYS: Final = 1
CSQ_PARENTHESES_UNNESTED: Final = 2
SNAKE_CASE_STEP1: Final[re.Pattern[str]]
SNAKE_CASE_STEP2: Final[re.Pattern[str]]
IDENTIFIER_RE: Final[re.Pattern[str]]
MODEL_BASE: Final = "_metaclass_helper_"

def make_identifier(s: str) -> str: ...
def chunked(it, n) -> Generator[list[Incomplete]]: ...

class _callable_context_manager:
    def __call__(self, fn): ...

class Proxy:
    __slots__ = ("obj", "_callbacks")
    def __init__(self) -> None: ...
    obj: Incomplete
    def initialize(self, obj) -> None: ...
    def attach_callback(self, callback): ...
    def passthrough(method): ...
    __enter__: Incomplete
    __exit__: Incomplete
    def __getattr__(self, attr: str): ...
    def __setattr__(self, attr: str, value) -> None: ...

class DatabaseProxy(Proxy):
    __slots__ = ("obj", "_callbacks", "_Model")
    def connection_context(self) -> ConnectionContext: ...
    def atomic(self, *args, **kwargs) -> _atomic: ...
    def manual_commit(self) -> _manual: ...
    def transaction(self, *args, **kwargs) -> _transaction: ...
    def savepoint(self) -> _savepoint: ...
    @property
    def Model(self) -> type[Model]: ...

class ModelDescriptor: ...

class AliasManager:
    __slots__ = ("_counter", "_current_index", "_mapping")
    def __init__(self) -> None: ...
    @property
    def mapping(self): ...
    def add(self, source): ...
    def get(self, source, any_depth: bool = False): ...
    def __getitem__(self, source): ...
    def __setitem__(self, source, alias) -> None: ...
    def push(self) -> None: ...
    def pop(self) -> None: ...

class State:
    def __new__(cls, scope=1, parentheses: bool = False, **kwargs) -> Self: ...
    def __call__(self, scope=None, parentheses=None, **kwargs) -> State: ...
    def __getattr__(self, attr_name: str): ...

class Context:
    __slots__ = ("stack", "_sql", "_values", "alias_manager", "state")
    stack: list[Incomplete]
    alias_manager: AliasManager
    state: State
    def __init__(self, **settings) -> None: ...
    def as_new(self) -> Context: ...
    def column_sort_key(self, item): ...
    @property
    def scope(self): ...
    @property
    def parentheses(self): ...
    @property
    def subquery(self): ...
    def __call__(self, **overrides) -> Self: ...
    scope_normal: Incomplete
    scope_source: Incomplete
    scope_values: Incomplete
    scope_cte: Incomplete
    scope_column: Incomplete
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...
    def push_alias(self) -> Generator[None]: ...
    def sql(self, obj): ...
    def literal(self, keyword) -> Self: ...
    def value(self, value, converter=None, add_param: bool = True): ...
    def __sql__(self, ctx): ...
    def parse(self, node): ...
    def query(self) -> tuple[str, list[Incomplete]]: ...

class Node:
    __isabstractmethod__: bool
    def clone(self) -> Self: ...
    def __sql__(self, ctx): ...
    @staticmethod
    def copy(method): ...
    def coerce(self, _coerce: bool = True) -> Self: ...
    def is_alias(self) -> bool: ...
    def unwrap(self) -> Self: ...

class ColumnFactory:
    __slots__ = ("node",)
    node: Node
    def __init__(self, node: Node) -> None: ...
    def __getattr__(self, attr: str) -> Column: ...
    __getitem__ = __getattr__

class _DynamicColumn:
    __slots__ = ()
    def __get__(self, instance, instance_type=None): ...

class _ExplicitColumn:
    __slots__ = ()
    def __get__(self, instance, instance_type=None) -> Self: ...

class Star(Node):
    def __init__(self, source) -> None: ...
    def __sql__(self, ctx): ...

class Source(Node):
    c: Incomplete
    def __init__(self, alias=None) -> None: ...
    def alias(self, name) -> Self: ...
    def select(self, *columns) -> Select: ...
    @property
    def __star__(self) -> Star: ...
    def join(self, dest, join_type="INNER JOIN", on=None) -> Join: ...
    def left_outer_join(self, dest, on=None) -> Join: ...
    def cte(self, name, recursive: bool = False, columns=None, materialized=None) -> CTE: ...
    def get_sort_key(self, ctx) -> tuple[Incomplete, ...]: ...
    def apply_alias(self, ctx): ...
    def apply_column(self, ctx): ...

class _HashableSource:
    def __init__(self, *args, **kwargs) -> None: ...
    def alias(self, name) -> Self: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other) -> Expression | bool: ...  # type: ignore[override]
    def __ne__(self, other) -> Expression | bool: ...  # type: ignore[override]
    __lt__: Callable[[Self, Any], Expression]
    __le__: Callable[[Self, Any], Expression]
    __gt__: Callable[[Self, Any], Expression]
    __ge__: Callable[[Self, Any], Expression]

class BaseTable(Source):
    def __and__(self, other) -> Join: ...
    def __add__(self, other) -> Join: ...
    def __sub__(self, other) -> Join: ...
    def __or__(self, other) -> Join: ...
    def __mul__(self, other) -> Join: ...
    def __rand__(self, other) -> Join: ...
    def __radd__(self, other) -> Join: ...
    def __rsub__(self, other) -> Join: ...
    def __ror__(self, other) -> Join: ...
    def __rmul__(self, other) -> Join: ...

class _BoundTableContext(_callable_context_manager):
    table: Incomplete
    database: Incomplete
    def __init__(self, table, database) -> None: ...
    def __enter__(self): ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class Table(_HashableSource, BaseTable):  # type: ignore[misc]
    __name__: Incomplete
    c: Incomplete
    primary_key: Incomplete
    def __init__(
        self, name, columns=None, primary_key=None, schema: str | None = None, alias=None, _model=None, _database=None
    ) -> None: ...
    def clone(self) -> Table: ...
    def bind(self, database=None) -> Self: ...
    def bind_ctx(self, database=None) -> _BoundTableContext: ...
    def select(self, *columns) -> Select: ...
    def insert(self, insert=None, columns=None, **kwargs) -> Insert: ...
    def replace(self, insert=None, columns=None, **kwargs): ...
    def update(self, update=None, **kwargs) -> Update: ...
    def delete(self) -> Delete: ...
    def __sql__(self, ctx): ...

class Join(BaseTable):
    lhs: Incomplete
    rhs: Incomplete
    join_type: Incomplete
    def __init__(self, lhs, rhs, join_type="INNER JOIN", on=None, alias=None) -> None: ...
    def on(self, predicate) -> Self: ...
    def __sql__(self, ctx): ...

class ValuesList(_HashableSource, BaseTable):  # type: ignore[misc]
    def __init__(self, values, columns=None, alias=None) -> None: ...
    def columns(self, *names) -> Self: ...
    def __sql__(self, ctx): ...

class CTE(_HashableSource, Source):  # type: ignore[misc]
    def __init__(self, name, query, recursive: bool = False, columns=None, materialized=None) -> None: ...
    def select_from(self, *columns): ...
    def union_all(self, rhs) -> CTE: ...
    __add__ = union_all
    def union(self, rhs) -> CTE: ...
    __or__ = union
    def __sql__(self, ctx): ...

class ColumnBase(Node):
    def converter(self, converter=None) -> Self: ...
    def alias(self, alias): ...
    def unalias(self) -> Self: ...
    def bind_to(self, dest) -> BindTo: ...
    def cast(self, as_type) -> Cast: ...
    def asc(self, collation=None, nulls=None) -> Ordering: ...
    __pos__ = asc
    def desc(self, collation=None, nulls=None) -> Ordering: ...
    __neg__ = desc
    def __invert__(self): ...
    __and__: Incomplete
    __or__: Incomplete
    def __add__(self, rhs: Any) -> Expression: ...
    __sub__: Incomplete
    __mul__: ClassVar[Callable[[Self, Any], Expression]]
    __div__: ClassVar[Callable[[Self, Any], Expression]]
    __truediv__: ClassVar[Callable[[Self, Any], Expression]]
    __xor__: ClassVar[Callable[[Self, Any], Expression]]
    def __radd__(self, rhs: Any) -> Expression: ...
    __rsub__: ClassVar[Callable[[Self, Any], Expression]]
    __rmul__: ClassVar[Callable[[Self, Any], Expression]]
    __rdiv__: ClassVar[Callable[[Self, Any], Expression]]
    __rtruediv__: ClassVar[Callable[[Self, Any], Expression]]
    __rand__: ClassVar[Callable[[Self, Any], Expression]]
    __ror__: ClassVar[Callable[[Self, Any], Expression]]
    __rxor__: ClassVar[Callable[[Self, Any], Expression]]
    def __eq__(self, rhs) -> Expression: ...  # type: ignore[override]
    def __ne__(self, rhs) -> Expression: ...  # type: ignore[override]
    __lt__: ClassVar[Callable[[Self, Any], Expression]]
    __le__: ClassVar[Callable[[Self, Any], Expression]]
    __gt__: ClassVar[Callable[[Self, Any], Expression]]
    __ge__: ClassVar[Callable[[Self, Any], Expression]]
    __lshift__: ClassVar[Callable[[Self, Any], Expression]]
    __rshift__: ClassVar[Callable[[Self, Any], Expression]]
    __mod__: ClassVar[Callable[[Self, Any], Expression]]
    __pow__: ClassVar[Callable[[Self, Any], Expression]]
    like: ClassVar[Callable[[Self, Any], Expression]]
    ilike: ClassVar[Callable[[Self, Any], Expression]]
    bin_and: ClassVar[Callable[[Self, Any], Expression]]
    bin_or: ClassVar[Callable[[Self, Any], Expression]]
    in_: ClassVar[Callable[[Self, Any], Expression]]
    not_in: ClassVar[Callable[[Self, Any], Expression]]
    regexp: ClassVar[Callable[[Self, Any], Expression]]
    iregexp: ClassVar[Callable[[Self, Any], Expression]]
    def is_null(self, is_null: bool = True) -> Expression: ...
    def contains(self, rhs) -> Expression: ...
    def startswith(self, rhs) -> Expression: ...
    def endswith(self, rhs) -> Expression: ...
    def between(self, lo, hi) -> Expression: ...
    def concat(self, rhs) -> StringExpression: ...
    def __getitem__(self, item): ...
    __iter__: Incomplete
    def distinct(self) -> NodeList: ...
    def collate(self, collation) -> NodeList: ...
    def get_sort_key(self, ctx) -> tuple[Incomplete, ...]: ...

class Column(ColumnBase):
    source: Incomplete
    name: Incomplete
    def __init__(self, source, name) -> None: ...
    def get_sort_key(self, ctx) -> tuple[Incomplete, ...]: ...
    def __hash__(self) -> int: ...
    def __sql__(self, ctx): ...

class WrappedNode(ColumnBase):
    node: Incomplete
    def __init__(self, node) -> None: ...
    def is_alias(self) -> bool: ...
    def unwrap(self): ...

class EntityFactory:
    __slots__ = ("node",)
    node: Incomplete
    def __init__(self, node) -> None: ...
    def __getattr__(self, attr: str) -> Entity: ...

class _DynamicEntity:
    __slots__ = ()
    def __get__(self, instance, instance_type=None): ...

class Alias(WrappedNode):
    c: Incomplete
    def __init__(self, node, alias) -> None: ...
    def __hash__(self) -> int: ...

    @property
    def name(self): ...
    @name.setter
    def name(self, value) -> None: ...

    def alias(self, alias=None): ...
    def unalias(self): ...
    def is_alias(self) -> bool: ...
    def __sql__(self, ctx): ...

class BindTo(WrappedNode):
    dest: Incomplete
    def __init__(self, node, dest) -> None: ...
    def __sql__(self, ctx): ...

class Negated(WrappedNode):
    def __invert__(self): ...
    def __sql__(self, ctx): ...

class BitwiseMixin:
    def __and__(self, other): ...
    def __or__(self, other): ...
    def __sub__(self, other): ...
    def __invert__(self) -> BitwiseNegated: ...

class BitwiseNegated(BitwiseMixin, WrappedNode):
    op: str
    def __invert__(self): ...
    def __sql__(self, ctx): ...

class Value(ColumnBase):
    value: Incomplete
    converter: Incomplete
    multi: bool
    values: list[Incomplete] | None
    def __init__(self, value, converter=None, unpack: bool = True) -> None: ...
    def __sql__(self, ctx): ...

class ValueLiterals(WrappedNode):
    def __sql__(self, ctx): ...

def AsIs(value, converter=None) -> Value: ...

class Cast(WrappedNode):
    def __init__(self, node, cast) -> None: ...
    def __sql__(self, ctx): ...

class Ordering(WrappedNode):
    direction: Incomplete
    collation: Incomplete
    nulls: Incomplete
    def __init__(self, node, direction, collation=None, nulls=None) -> None: ...
    def collate(self, collation=None) -> Ordering: ...  # type: ignore[override]
    def __sql__(self, ctx): ...

class Expression(ColumnBase):
    lhs: Incomplete
    op: Incomplete
    rhs: Incomplete
    flat: bool
    def __init__(self, lhs, op, rhs, flat: bool = False) -> None: ...
    def __sql__(self, ctx): ...

class StringExpression(Expression):
    def __add__(self, rhs) -> StringExpression: ...
    def __radd__(self, lhs) -> StringExpression: ...

class Entity(ColumnBase):
    def __init__(self, *path) -> None: ...
    def __getattr__(self, attr: str) -> Entity: ...
    def get_sort_key(self, ctx) -> tuple[Incomplete, ...]: ...
    def __hash__(self) -> int: ...
    def __sql__(self, ctx): ...

class SQL(ColumnBase):
    sql: Incomplete
    params: Incomplete
    def __init__(self, sql, params=None) -> None: ...
    def __sql__(self, ctx): ...

def Check(constraint, name=None): ...
def Default(value) -> SQL: ...

class Function(ColumnBase):
    no_coerce_functions: ClassVar[set[str]]
    name: Incomplete
    arguments: Incomplete
    def __init__(self, name, arguments, coerce: bool = True, python_value=None) -> None: ...
    def __getattr__(self, attr: str): ...
    def filter(self, where=None) -> Self: ...
    def order_by(self, *ordering) -> Self: ...
    def python_value(self, func=None) -> Self: ...
    def over(
        self, partition_by=None, order_by=None, start=None, end=None, frame_type=None, window=None, exclude=None
    ) -> NodeList: ...
    def __sql__(self, ctx): ...

fn: Incomplete

class Window(Node):
    CURRENT_ROW: SQL
    GROUP: SQL
    TIES: SQL
    NO_OTHERS: SQL
    GROUPS: str
    RANGE: str
    ROWS: str
    partition_by: Incomplete
    order_by: Incomplete
    start: Incomplete
    end: Incomplete
    frame_type: Incomplete
    def __init__(
        self,
        partition_by=None,
        order_by=None,
        start=None,
        end=None,
        frame_type=None,
        extends=None,
        exclude=None,
        alias=None,
        _inline: bool = False,
    ) -> None: ...
    def alias(self, alias=None) -> Self: ...
    def as_range(self) -> Self: ...
    def as_rows(self) -> Self: ...
    def as_groups(self) -> Self: ...
    def extends(self, window=None) -> Self: ...
    def exclude(self, frame_exclusion=None) -> Self: ...
    @staticmethod
    def following(value=None) -> SQL: ...
    @staticmethod
    def preceding(value=None) -> SQL: ...
    def __sql__(self, ctx): ...

class WindowAlias(Node):
    window: Incomplete
    def __init__(self, window) -> None: ...
    def alias(self, window_alias) -> Self: ...
    def __sql__(self, ctx): ...

class ForUpdate(Node):
    def __init__(self, expr, of=None, nowait: bool | None = None, skip_locked: bool | None = None) -> None: ...
    def __sql__(self, ctx): ...

class Case(ColumnBase):
    predicate: Incomplete
    expression_tuples: Incomplete
    default: Incomplete | None
    def __init__(self, predicate, expression_tuples, default=None) -> None: ...
    def __sql__(self, ctx): ...

class NodeList(ColumnBase):
    nodes: Incomplete
    glue: Incomplete
    parens: bool
    def __init__(self, nodes, glue: str = " ", parens: bool = False) -> None: ...
    def __sql__(self, ctx): ...

class _Namespace(Node):
    __slots__ = ("_name",)
    def __init__(self, name) -> None: ...
    def __getattr__(self, attr: str) -> NamespaceAttribute: ...
    __getitem__ = __getattr__

class NamespaceAttribute(ColumnBase):
    def __init__(self, namespace, attribute) -> None: ...
    def __sql__(self, ctx): ...

EXCLUDED: Incomplete

class DQ(ColumnBase):
    query: Incomplete
    def __init__(self, **query) -> None: ...
    def __invert__(self) -> Self: ...  # type: ignore[override]
    def clone(self) -> DQ: ...

Tuple: Incomplete

class QualifiedNames(WrappedNode):
    def __sql__(self, ctx): ...

class OnConflict(Node):
    def __init__(
        self,
        action=None,
        update=None,
        preserve=None,
        where=None,
        conflict_target=None,
        conflict_where=None,
        conflict_constraint=None,
    ) -> None: ...
    def get_conflict_statement(self, ctx, query): ...
    def get_conflict_update(self, ctx, query): ...
    def preserve(self, *columns) -> Self: ...
    def update(self, _data=None, **kwargs) -> Self: ...
    def where(self, *expressions) -> Self: ...
    def conflict_target(self, *constraints) -> Self: ...
    def conflict_where(self, *expressions) -> Self: ...
    def conflict_constraint(self, constraint) -> Self: ...

class BaseQuery(Node):
    default_row_type: Incomplete
    def __init__(self, _database=None, **kwargs) -> None: ...
    def bind(self, database=None) -> Self: ...
    def clone(self) -> Self: ...
    def dicts(self, as_dict: bool = True) -> Self: ...
    def tuples(self, as_tuple: bool = True) -> Self: ...
    def namedtuples(self, as_namedtuple: bool = True) -> Self: ...
    def objects(self, constructor=None) -> Self: ...
    def __sql__(self, ctx) -> None: ...
    def sql(self): ...
    def execute(self, database=None): ...
    def iterator(self, database=None): ...
    def __iter__(self): ...
    def __getitem__(self, value): ...
    def __len__(self) -> int: ...

class RawQuery(BaseQuery):
    def __init__(self, sql=None, params=None, **kwargs) -> None: ...
    def __sql__(self, ctx): ...

class Query(BaseQuery):
    def __init__(self, where=None, order_by=None, limit=None, offset=None, **kwargs) -> None: ...
    def with_cte(self, *cte_list) -> Self: ...
    def where(self, *expressions) -> Self: ...
    def orwhere(self, *expressions) -> Self: ...
    def order_by(self, *values) -> Self: ...
    def order_by_extend(self, *values) -> Self: ...
    def limit(self, value=None) -> Self: ...
    def offset(self, value=None) -> Self: ...
    def paginate(self, page, paginate_by: int = 20) -> Self: ...
    def __sql__(self, ctx): ...

class SelectQuery(Query):
    def union_all(self, other) -> CompoundSelectQuery: ...
    def __add__(self, other) -> CompoundSelectQuery: ...
    def union(self, other) -> CompoundSelectQuery: ...
    def __or__(self, other) -> CompoundSelectQuery: ...
    def intersect(self, other) -> CompoundSelectQuery: ...
    def __and__(self, other) -> CompoundSelectQuery: ...
    def except_(self, other) -> CompoundSelectQuery: ...
    def __sub__(self, other) -> CompoundSelectQuery: ...
    def __radd__(self, other) -> CompoundSelectQuery: ...
    def __ror__(self, other) -> CompoundSelectQuery: ...
    def __rand__(self, other) -> CompoundSelectQuery: ...
    def __rsub__(self, other) -> CompoundSelectQuery: ...
    def select_from(self, *columns) -> Select: ...

class SelectBase(_HashableSource, Source, SelectQuery):  # type: ignore[misc]
    def peek(self, database=None, n: int = 1): ...
    def first(self, database=None, n: int = 1): ...
    def scalar(self, database=None, as_tuple: bool = False, as_dict: bool = False): ...
    def scalars(self, database=None) -> Generator[Incomplete]: ...
    def count(self, database=None, clear_limit: bool = False) -> int: ...
    def exists(self, database=None) -> bool: ...
    def get(self, database=None): ...

class CompoundSelectQuery(SelectBase):
    lhs: Incomplete
    op: Incomplete
    rhs: Incomplete
    def __init__(self, lhs, op, rhs) -> None: ...
    def exists(self, database=None) -> bool: ...
    def __sql__(self, ctx): ...

class Select(SelectBase):
    def __init__(
        self,
        from_list=None,
        columns=None,
        group_by=None,
        having=None,
        distinct=None,
        windows=None,
        for_update=None,
        lateral=None,
        **kwargs,
    ) -> None: ...
    def clone(self) -> Self: ...
    def columns(self, *columns, **kwargs) -> Self: ...
    select = columns
    def select_extend(self, *columns) -> Self: ...

    @property
    def selected_columns(self): ...
    @selected_columns.setter
    def selected_columns(self, value) -> None: ...

    def from_(self, *sources) -> Self: ...
    def join(self, dest, join_type="INNER JOIN", on=None) -> Self: ...  # type: ignore[override]
    def left_outer_join(self, dest, on=None) -> Self: ...  # type: ignore[override]
    def group_by(self, *columns) -> Self: ...
    def group_by_extend(self, *values) -> Self: ...
    def having(self, *expressions) -> Self: ...
    def distinct(self, *columns) -> Self: ...
    def window(self, *windows) -> Self: ...
    def for_update(
        self, for_update: bool = True, of=None, nowait: bool | None = None, skip_locked: bool | None = None
    ) -> Self: ...
    def lateral(self, lateral: bool = True) -> Self: ...
    def __sql_selection__(self, ctx, is_subquery: bool = False): ...
    def __sql__(self, ctx): ...

class _WriteQuery(Query):
    table: Incomplete
    def __init__(self, table, returning=None, **kwargs) -> None: ...
    def cte(self, name, recursive: bool = False, columns=None, materialized=None) -> CTE: ...
    def returning(self, *returning) -> Self: ...
    def apply_returning(self, ctx): ...
    def execute_returning(self, database): ...
    def handle_result(self, database, cursor): ...
    def __sql__(self, ctx): ...

class Update(_WriteQuery):
    def __init__(self, table, update=None, **kwargs) -> None: ...
    def from_(self, *sources) -> Self: ...
    def __sql__(self, ctx): ...

class Insert(_WriteQuery):
    SIMPLE: int
    QUERY: int
    MULTI: int

    class DefaultValuesException(Exception): ...

    def __init__(self, table, insert=None, columns=None, on_conflict=None, **kwargs) -> None: ...
    def where(self, *expressions): ...
    def as_rowcount(self, _as_rowcount: bool = True) -> Self: ...
    def on_conflict_ignore(self, ignore: bool = True) -> Self: ...
    def on_conflict_replace(self, replace: bool = True) -> Self: ...
    def on_conflict(self, *args, **kwargs) -> Self: ...
    def get_default_data(self): ...
    def get_default_columns(self) -> list[Incomplete] | None: ...
    def __sql__(self, ctx): ...
    def handle_result(self, database, cursor): ...

class Delete(_WriteQuery):
    def __sql__(self, ctx): ...

class Index(Node):
    def __init__(
        self, name, table, expressions, unique: bool = False, safe: bool = False, where=None, using=None, nulls_distinct=None
    ) -> None: ...
    def safe(self, _safe: bool = True) -> Self: ...
    def where(self, *expressions) -> Self: ...
    def using(self, _using=None) -> Self: ...
    def nulls_distinct(self, nulls_distinct=None) -> Self: ...
    def __sql__(self, ctx): ...

class ModelIndex(Index):
    def __init__(
        self, model, fields, unique: bool = False, safe: bool = True, where=None, using=None, name=None, nulls_distinct=None
    ) -> None: ...

class PeeweeException(Exception):
    def __init__(self, *args) -> None: ...

class ImproperlyConfigured(PeeweeException): ...
class DatabaseError(PeeweeException): ...
class DataError(DatabaseError): ...
class IntegrityError(DatabaseError): ...
class InterfaceError(PeeweeException): ...
class InternalError(DatabaseError): ...
class NotSupportedError(DatabaseError): ...
class OperationalError(DatabaseError): ...
class ProgrammingError(DatabaseError): ...

class ExceptionWrapper:
    __slots__ = ("exceptions",)
    exceptions: Incomplete
    def __init__(self, exceptions) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...

class IndexMetadata(NamedTuple):
    name: Incomplete
    sql: Incomplete
    columns: Incomplete
    unique: Incomplete
    table: Incomplete

class ColumnMetadata(NamedTuple):
    name: Incomplete
    data_type: Incomplete
    null: Incomplete
    primary_key: Incomplete
    table: Incomplete
    default: Incomplete

class ForeignKeyMetadata(NamedTuple):
    column: Incomplete
    dest_table: Incomplete
    dest_column: Incomplete
    table: Incomplete

class ViewMetadata(NamedTuple):
    name: Incomplete
    sql: Incomplete

class _ConnectionState:
    def __init__(self, **kwargs) -> None: ...
    closed: bool
    conn: Incomplete
    ctx: Incomplete
    transactions: Incomplete
    def reset(self) -> None: ...
    def set_connection(self, conn) -> None: ...

class _ConnectionLocal(_ConnectionState, threading.local): ...

class _NoopLock:
    __slots__ = ()
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class ConnectionContext(_callable_context_manager):
    __slots__ = ("db",)
    db: Incomplete
    def __init__(self, db) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class Database(_callable_context_manager):
    context_class: Incomplete
    field_types: Incomplete
    operations: Incomplete
    param: str
    quote: str
    server_version: Incomplete
    commit_select: bool
    compound_select_parentheses: Incomplete
    for_update: bool
    index_schema_prefix: bool
    index_using_precedes_table: bool
    limit_max: Incomplete
    nulls_ordering: bool
    returning_clause: bool
    safe_create_index: bool
    safe_drop_index: bool
    sequences: bool
    truncate_table: bool
    autoconnect: Incomplete
    thread_safe: Incomplete
    connect_params: Incomplete
    def __init__(
        self,
        database,
        thread_safe: bool = True,
        autorollback: bool = False,
        field_types=None,
        operations=None,
        autocommit=None,
        autoconnect: bool = True,
        **kwargs,
    ) -> None: ...
    database: Incomplete
    deferred: Incomplete
    def init(self, database, **kwargs) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...
    def connection_context(self) -> ConnectionContext: ...
    def connect(self, reuse_if_open: bool = False) -> bool: ...
    def close(self) -> bool: ...
    def is_closed(self) -> bool: ...
    def is_connection_usable(self) -> bool: ...
    def connection(self): ...
    def cursor(self, named_cursor=None): ...
    def execute_sql(self, sql, params=None): ...
    def execute(self, query, **context_options): ...
    def get_context_options(self) -> dict[str, Incomplete]: ...
    def get_sql_context(self, **context_options) -> context_class: ...
    def conflict_statement(self, on_conflict, query): ...
    def conflict_update(self, on_conflict, query): ...
    def last_insert_id(self, cursor, query_type=None): ...
    def rows_affected(self, cursor): ...
    def default_values_insert(self, ctx): ...
    def session_start(self) -> _transaction: ...
    def session_commit(self) -> bool: ...
    def session_rollback(self) -> bool: ...
    def in_transaction(self) -> bool: ...
    def push_transaction(self, transaction) -> None: ...
    def pop_transaction(self): ...
    def transaction_depth(self) -> int: ...
    def top_transaction(self): ...
    def atomic(self, *args, **kwargs) -> _atomic: ...
    def manual_commit(self) -> _manual: ...
    def transaction(self, *args, **kwargs) -> _transaction: ...
    def savepoint(self) -> _savepoint: ...
    def begin(self) -> None: ...
    def commit(self) -> None: ...
    def rollback(self) -> None: ...
    def batch_commit(self, it, n) -> Generator[Incomplete]: ...
    def table_exists(self, table_name, schema: str | None = None) -> bool: ...
    def get_tables(self, schema: str | None = None) -> list[str]: ...
    def get_indexes(self, table, schema: str | None = None) -> list[IndexMetadata]: ...
    def get_columns(self, table, schema: str | None = None) -> list[ColumnMetadata]: ...
    def get_primary_keys(self, table, schema: str | None = None): ...
    def get_foreign_keys(self, table, schema: str | None = None) -> list[ForeignKeyMetadata]: ...
    def sequence_exists(self, seq) -> bool: ...
    def create_tables(self, models: Iterable[type[Model]], **options) -> None: ...
    def drop_tables(self, models: Iterable[type[Model]], **kwargs) -> None: ...
    def extract_date(self, date_part, date_field): ...
    def truncate_date(self, date_part, date_field): ...
    def to_timestamp(self, date_field): ...
    def from_timestamp(self, date_field): ...
    def random(self): ...
    def bind(self, models: Iterable[type[Model]], bind_refs: bool = True, bind_backrefs: bool = True) -> None: ...
    def bind_ctx(
        self, models: Iterable[type[Model]], bind_refs: bool = True, bind_backrefs: bool = True
    ) -> _BoundModelsContext: ...
    def get_noop_select(self, ctx): ...
    @property
    def Model(self) -> type[Model]: ...

class SqliteDatabase(Database):
    field_types: Incomplete
    operations: Incomplete
    index_schema_prefix: bool
    limit_max: int
    server_version: Incomplete
    truncate_table: bool
    nulls_ordering: bool
    def __init__(
        self, database, pragmas=None, regexp_function: bool = False, rank_functions: bool = False, *args, **kwargs
    ) -> None: ...
    returning_clause: Incomplete
    def init(self, database, pragmas=None, timeout: int = 5, returning_clause=None, **kwargs) -> None: ...
    def pragma(self, key, value=..., permanent: bool = False, schema: str | None = None): ...
    cache_size: Incomplete
    foreign_keys: Incomplete
    journal_mode: Incomplete
    journal_size_limit: Incomplete
    mmap_size: Incomplete
    page_size: Incomplete
    read_uncommitted: Incomplete
    synchronous: Incomplete
    wal_autocheckpoint: Incomplete
    application_id: Incomplete
    user_version: Incomplete
    data_version: Incomplete

    @property
    def timeout(self): ...
    @timeout.setter
    def timeout(self, seconds) -> None: ...

    def register_aggregate(self, klass, name=None, num_params: int = -1) -> None: ...
    def aggregate(self, name=None, num_params: int = -1): ...
    def register_collation(self, fn, name=None) -> None: ...
    def collation(self, name=None): ...
    def register_function(self, fn, name: str | None = None, num_params: int = -1, deterministic: bool | None = None) -> None: ...
    def func(self, name: str | None = None, num_params: int = -1, deterministic: bool | None = None) -> Callable[[_F], _F]: ...
    def register_window_function(self, klass, name=None, num_params: int = -1) -> None: ...
    def window_function(self, name=None, num_params: int = -1): ...
    def unregister_aggregate(self, name) -> None: ...
    def unregister_collation(self, name) -> None: ...
    def unregister_function(self, name) -> None: ...
    def unregister_window_function(self, name) -> None: ...
    def load_extension(self, extension) -> None: ...
    def unload_extension(self, extension) -> None: ...
    def attach(self, filename, name) -> bool: ...
    def detach(self, name) -> bool: ...
    def last_insert_id(self, cursor, query_type=None): ...
    def rows_affected(self, cursor): ...
    def begin(self, lock_type=None) -> None: ...
    def get_tables(self, schema: str | None = None) -> list[str]: ...
    def get_views(self, schema: str | None = None) -> list[ViewMetadata]: ...
    def get_indexes(self, table, schema: str | None = None) -> list[IndexMetadata]: ...
    def get_columns(self, table, schema: str | None = None) -> list[ColumnMetadata]: ...
    def get_primary_keys(self, table, schema: str | None = None) -> list[Incomplete]: ...
    def get_foreign_keys(self, table, schema: str | None = None) -> list[ForeignKeyMetadata]: ...
    def get_binary_type(self): ...
    def conflict_statement(self, on_conflict, query) -> SQL | None: ...
    def conflict_update(self, oc, query) -> SQL | NodeList | None: ...
    def extract_date(self, date_part, date_field) -> Function: ...
    def truncate_date(self, date_part, date_field) -> Function: ...
    def to_timestamp(self, date_field) -> Cast: ...
    def from_timestamp(self, date_field) -> Function: ...

class _BasePsycopgAdapter:
    isolation_levels: dict[int, str]
    isolation_levels_inv: dict[str, int]
    def __init__(self) -> None: ...

    @overload
    def isolation_level_int(self, isolation_level: str) -> int: ...
    @overload
    def isolation_level_int(self, isolation_level: _T) -> _T: ...

    @overload
    def isolation_level_str(self, isolation_level: int) -> str: ...
    @overload
    def isolation_level_str(self, isolation_level: _T) -> _T: ...

class Psycopg2Adapter(_BasePsycopgAdapter):
    json_type: Incomplete
    jsonb_type: Incomplete
    cast_json_case: bool
    def __init__(self) -> None: ...
    def check_driver(self) -> None: ...
    def get_binary_type(self) -> type[Incomplete]: ...
    def connect(self, db, **params): ...
    def get_server_version(self, conn): ...
    def is_connection_usable(self, conn) -> bool: ...
    def is_connection_reusable(self, conn) -> bool: ...
    def is_connection_closed(self, conn) -> bool: ...

class Psycopg3Adapter(_BasePsycopgAdapter):
    json_type: Incomplete
    jsonb_type: Incomplete
    cast_json_case: bool
    def __init__(self) -> None: ...
    def check_driver(self) -> None: ...
    def get_binary_type(self) -> type[Incomplete]: ...
    def connect(self, db, **params): ...
    def get_server_version(self, conn): ...
    def is_connection_usable(self, conn) -> bool: ...
    def is_connection_reusable(self, conn) -> bool: ...
    def is_connection_closed(self, conn) -> bool: ...

class PostgresqlDatabase(Database):
    field_types: Incomplete
    operations: Incomplete
    param: str
    commit_select: bool
    compound_select_parentheses: Incomplete
    for_update: bool
    nulls_ordering: bool
    returning_clause: bool
    sequences: bool
    psycopg2_adapter: Incomplete
    psycopg3_adapter: Incomplete
    def init(
        self,
        database,
        register_unicode: bool = True,
        encoding=None,
        isolation_level=None,
        *,
        prefer_psycopg3: bool = False,
        **kwargs,
    ) -> None: ...
    def is_connection_usable(self) -> bool: ...
    def last_insert_id(self, cursor, query_type=None): ...
    def rows_affected(self, cursor): ...
    def begin(self, isolation_level: str | None = None) -> None: ...
    def get_tables(self, schema: str | None = None) -> list[str]: ...
    def get_views(self, schema: str | None = None) -> list[ViewMetadata]: ...
    def get_indexes(self, table, schema: str | None = None) -> list[IndexMetadata]: ...
    def get_columns(self, table, schema: str | None = None) -> list[ColumnMetadata]: ...
    def get_primary_keys(self, table, schema: str | None = None) -> list[Incomplete]: ...
    def get_foreign_keys(self, table, schema: str | None = None) -> list[ForeignKeyMetadata]: ...
    def sequence_exists(self, sequence) -> bool: ...
    def get_binary_type(self) -> type[Incomplete]: ...
    def conflict_statement(self, on_conflict, query) -> None: ...
    def conflict_update(self, oc, query) -> NodeList: ...
    def extract_date(self, date_part, date_field) -> Function: ...
    def truncate_date(self, date_part, date_field) -> Function: ...
    def interval(self, val) -> NodeList: ...
    def to_timestamp(self, date_field) -> Function: ...
    def from_timestamp(self, date_field) -> Function: ...
    def get_noop_select(self, ctx): ...
    def set_time_zone(self, timezone) -> None: ...
    def set_isolation_level(self, isolation_level: int | str) -> None: ...

class MySQLDatabase(Database):
    field_types: Incomplete
    operations: Incomplete
    param: str
    quote: str
    commit_select: bool
    compound_select_parentheses: Incomplete
    for_update: bool
    index_using_precedes_table: bool
    limit_max: Incomplete
    safe_create_index: bool
    safe_drop_index: bool
    sql_mode: str
    def init(self, database, **kwargs) -> None: ...
    def is_connection_usable(self) -> bool: ...
    def default_values_insert(self, ctx): ...
    def begin(self, isolation_level: str | None = None) -> None: ...
    def get_tables(self, schema: str | None = None) -> list[str]: ...
    def get_views(self, schema: str | None = None) -> list[ViewMetadata]: ...
    def get_indexes(self, table, schema: str | None = None) -> list[IndexMetadata]: ...
    def get_columns(self, table, schema: str | None = None) -> list[ColumnMetadata]: ...
    def get_primary_keys(self, table, schema: str | None = None) -> list[Incomplete]: ...
    def get_foreign_keys(self, table, schema: str | None = None) -> list[ForeignKeyMetadata]: ...
    def get_binary_type(self): ...
    def conflict_statement(self, on_conflict, query) -> SQL | None: ...
    def conflict_update(self, on_conflict, query) -> NodeList | None: ...
    def extract_date(self, date_part, date_field) -> Function: ...
    def truncate_date(self, date_part, date_field) -> Function: ...
    def to_timestamp(self, date_field) -> Function: ...
    def from_timestamp(self, date_field) -> Function: ...
    def random(self) -> Function: ...
    def get_noop_select(self, ctx): ...

class _manual(_callable_context_manager):
    db: Incomplete
    def __init__(self, db) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class _atomic(_callable_context_manager):
    db: Incomplete
    def __init__(self, db, *args, **kwargs) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None): ...

class _transaction(_callable_context_manager):
    db: Incomplete
    def __init__(self, db, *args, **kwargs) -> None: ...
    def commit(self, begin: bool = True) -> None: ...
    def rollback(self, begin: bool = True) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class _savepoint(_callable_context_manager):
    db: Incomplete
    sid: Incomplete
    quoted_sid: Incomplete
    def __init__(self, db, sid=None) -> None: ...
    def commit(self, begin: bool = True) -> None: ...
    def rollback(self, begin: bool = True) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class CursorWrapper:
    cursor: Incomplete
    count: int
    index: int
    initialized: bool
    populated: bool
    row_cache: list[Incomplete]
    def __init__(self, cursor) -> None: ...
    def __iter__(self): ...
    def __getitem__(self, item): ...
    def __len__(self) -> int: ...
    def initialize(self) -> None: ...
    def iterate(self, cache: bool = True): ...
    def process_row(self, row): ...
    def iterator(self) -> Generator[Incomplete]: ...
    def fill_cache(self, n: int = 0) -> None: ...
    def dedupe_columns(self, columns: Iterable[str], valid_identifiers: bool = True) -> list[str]: ...

class DictCursorWrapper(CursorWrapper):
    columns: list[str]
    ncols: int
    def initialize(self) -> None: ...
    def process_row(self, row): ...

class NamedTupleCursorWrapper(CursorWrapper):
    tuple_class: Incomplete
    def initialize(self) -> None: ...
    def process_row(self, row): ...

class ObjectCursorWrapper(DictCursorWrapper):
    constructor: Incomplete
    columns: list[str]
    ncols: int
    def __init__(self, cursor, constructor) -> None: ...
    def initialize(self) -> None: ...
    def process_row(self, row): ...

class ResultIterator:
    cursor_wrapper: Incomplete
    index: int
    def __init__(self, cursor_wrapper) -> None: ...
    def __iter__(self) -> Self: ...
    def next(self): ...
    __next__ = next

class FieldAccessor:
    model: Incomplete
    field: Incomplete
    name: Incomplete
    def __init__(self, model, field, name) -> None: ...
    def __get__(self, instance, instance_type=None): ...
    def __set__(self, instance, value) -> None: ...

class ForeignKeyAccessor(FieldAccessor):
    rel_model: Incomplete
    def __init__(self, model, field, name) -> None: ...
    def get_rel_instance(self, instance): ...
    def __get__(self, instance, instance_type=None): ...
    def __set__(self, instance, obj) -> None: ...

class BackrefAccessor:
    field: Incomplete
    model: Incomplete
    rel_model: Incomplete
    def __init__(self, field) -> None: ...
    def __get__(self, instance, instance_type=None): ...

class ObjectIdAccessor:
    field: Incomplete
    def __init__(self, field) -> None: ...
    def __get__(self, instance, instance_type=None): ...
    def __set__(self, instance, value) -> None: ...

class Field(ColumnBase):
    accessor_class: Incomplete
    auto_increment: bool
    default_index_type: Incomplete
    field_type: ClassVar[str]
    unpack: bool
    null: Incomplete
    index: Incomplete
    unique: Incomplete
    column_name: Incomplete
    default: Incomplete
    primary_key: Incomplete
    constraints: Incomplete
    sequence: Incomplete
    collation: Incomplete
    unindexed: Incomplete
    choices: Incomplete
    help_text: Incomplete
    verbose_name: Incomplete
    index_type: Incomplete
    def __init__(
        self,
        null: bool = False,
        index: bool = False,
        unique: bool = False,
        column_name=None,
        default=None,
        primary_key: bool = False,
        constraints=None,
        sequence=None,
        collation=None,
        unindexed: bool = False,
        choices=None,
        help_text=None,
        verbose_name=None,
        index_type=None,
        db_column=None,
        _hidden: bool = False,
    ) -> None: ...
    def __hash__(self) -> int: ...
    model: Incomplete
    name: Incomplete
    def bind(self, model, name, set_attribute: bool = True) -> None: ...
    @property
    def column(self) -> Column: ...
    def adapt(self, value): ...
    def db_value(self, value): ...
    def python_value(self, value): ...
    def to_value(self, value, case: bool = False) -> Value: ...
    def get_sort_key(self, ctx) -> tuple[Incomplete, ...]: ...
    def __sql__(self, ctx): ...
    def get_modifiers(self) -> list[int] | None: ...
    def ddl_datatype(self, ctx) -> SQL: ...
    def ddl(self, ctx) -> NodeList: ...

class AnyField(Field): ...

class IntegerField(Field):
    def adapt(self, value): ...

class BigIntegerField(IntegerField): ...
class SmallIntegerField(IntegerField): ...

class AutoField(IntegerField):
    auto_increment: bool
    def __init__(self, *args, **kwargs) -> None: ...

class BigAutoField(AutoField): ...

class IdentityField(AutoField):
    def __init__(self, generate_always: bool = False, **kwargs) -> None: ...

class PrimaryKeyField(AutoField):
    def __init__(self, *args, **kwargs) -> None: ...

class FloatField(Field):
    def adapt(self, value): ...

class DoubleField(FloatField): ...

class DecimalField(Field):
    max_digits: Incomplete
    decimal_places: Incomplete
    auto_round: Incomplete
    rounding: Incomplete
    def __init__(
        self, max_digits: int = 10, decimal_places: int = 5, auto_round: bool = False, rounding=None, *args, **kwargs
    ) -> None: ...
    def get_modifiers(self) -> list[int]: ...
    def db_value(self, value): ...
    def python_value(self, value) -> Decimal | None: ...

class _StringField(Field):
    def adapt(self, value) -> str: ...
    def __add__(self, other) -> StringExpression: ...
    def __radd__(self, other) -> StringExpression: ...

class CharField(_StringField):
    max_length: int
    def __init__(self, max_length: int = 255, *args, **kwargs) -> None: ...
    def get_modifiers(self) -> list[int] | None: ...

class FixedCharField(CharField):
    def adapt(self, value) -> str: ...

class TextField(_StringField): ...

class FieldDatabaseHook:
    def bind(self, model, name, set_attribute: bool = True): ...

class BlobField(FieldDatabaseHook, Field):
    def db_value(self, value): ...

class BitField(BitwiseMixin, BigIntegerField):
    def __init__(self, *args, **kwargs) -> None: ...
    def flag(self, value=None): ...

class BigBitFieldData:
    instance: Incomplete
    name: Incomplete
    def __init__(self, instance, name) -> None: ...
    def clear(self) -> None: ...
    def set_bit(self, idx) -> None: ...
    def clear_bit(self, idx) -> None: ...
    def toggle_bit(self, idx) -> bool: ...
    def is_set(self, idx) -> bool: ...
    __getitem__ = is_set
    def __setitem__(self, item: int, value: bool) -> None: ...
    __delitem__ = clear_bit
    def __len__(self) -> int: ...
    def __and__(self, other: BigBitFieldData | bytes | bytearray | memoryview) -> bytearray: ...
    def __or__(self, other: BigBitFieldData | bytes | bytearray | memoryview) -> bytearray: ...
    def __xor__(self, other: BigBitFieldData | bytes | bytearray | memoryview) -> bytearray: ...
    def __iter__(self) -> Iterator[Literal[0, 1]]: ...
    def __bytes__(self) -> bytes: ...

class BigBitFieldAccessor(FieldAccessor):
    def __get__(self, instance, instance_type=None): ...
    def __set__(self, instance, value) -> None: ...

class BigBitField(BlobField):
    accessor_class: Incomplete
    def __init__(self, *args, **kwargs) -> None: ...
    def db_value(self, value): ...

class UUIDField(Field):
    def db_value(self, value): ...
    def python_value(self, value) -> UUID | None: ...

class BinaryUUIDField(BlobField):
    def db_value(self, value): ...
    def python_value(self, value) -> UUID | None: ...

class _BaseFormattedField(Field):
    formats: Incomplete
    def __init__(self, formats=None, *args, **kwargs) -> None: ...

class DateTimeField(_BaseFormattedField):
    formats: Incomplete
    def adapt(self, value): ...
    def to_timestamp(self): ...
    def truncate(self, part): ...
    @property
    def year(self): ...
    @property
    def month(self): ...
    @property
    def day(self): ...
    @property
    def hour(self): ...
    @property
    def minute(self): ...
    @property
    def second(self): ...

class DateField(_BaseFormattedField):
    formats: Incomplete
    def adapt(self, value): ...
    def to_timestamp(self): ...
    def truncate(self, part): ...
    @property
    def year(self): ...
    @property
    def month(self): ...
    @property
    def day(self): ...

class TimeField(_BaseFormattedField):
    formats: Incomplete
    def adapt(self, value): ...
    @property
    def hour(self): ...
    @property
    def minute(self): ...
    @property
    def second(self): ...

class TimestampField(BigIntegerField):
    valid_resolutions: Incomplete
    resolution: Incomplete
    ticks_to_microsecond: Incomplete
    utc: Incomplete
    def __init__(self, *args, **kwargs) -> None: ...
    def local_to_utc(self, dt) -> datetime: ...
    def utc_to_local(self, dt) -> datetime: ...
    def get_timestamp(self, value) -> float: ...
    def db_value(self, value) -> int | None: ...
    def python_value(self, value): ...
    def from_timestamp(self): ...
    @property
    def year(self): ...
    @property
    def month(self): ...
    @property
    def day(self): ...
    @property
    def hour(self): ...
    @property
    def minute(self): ...
    @property
    def second(self): ...

class IPField(BigIntegerField):
    def db_value(self, val): ...
    def python_value(self, val) -> str | None: ...

class BooleanField(Field):
    adapt: Incomplete

class BareField(Field):
    adapt: Incomplete
    def __init__(self, adapt=None, *args, **kwargs) -> None: ...
    def ddl_datatype(self, ctx): ...

class ForeignKeyField(Field):
    accessor_class: Incomplete
    backref_accessor_class: Incomplete
    rel_model: Incomplete
    rel_field: Incomplete
    declared_backref: Incomplete
    backref: Incomplete
    on_delete: Incomplete
    on_update: Incomplete
    deferrable: Incomplete
    deferred: Incomplete
    object_id_name: Incomplete
    lazy_load: Incomplete
    constraint_name: str | None
    def __init__(
        self,
        model,
        field=None,
        backref=None,
        on_delete=None,
        on_update=None,
        deferrable=None,
        _deferred=None,
        rel_model=None,
        to_field=None,
        object_id_name=None,
        lazy_load: bool = True,
        constraint_name: str | None = None,
        related_name=None,
        *args,
        **kwargs,
    ) -> None: ...
    @property
    def field_type(self): ...  # type: ignore[override]
    def get_modifiers(self): ...
    def adapt(self, value): ...
    def db_value(self, value): ...
    def python_value(self, value): ...
    column_name: Incomplete
    safe_name: Incomplete
    def bind(self, model, name, set_attribute: bool = True) -> None: ...
    def foreign_key_constraint(self, explicit_name: bool = False) -> NodeList: ...
    def get_constraint_name(self) -> str: ...
    def __getattr__(self, attr: str): ...

class DeferredForeignKey(Field):
    field_kwargs: Incomplete
    rel_model_name: Incomplete
    def __init__(self, rel_model_name, **kwargs) -> None: ...
    __hash__ = object.__hash__
    def __deepcopy__(self, memo=None) -> DeferredForeignKey: ...
    def set_model(self, rel_model) -> None: ...
    @staticmethod
    def resolve(model_cls) -> None: ...

class DeferredThroughModel:
    def __init__(self) -> None: ...
    def set_field(self, model, field, name) -> None: ...
    def set_model(self, through_model) -> None: ...

class MetaField(Field):
    column_name: Incomplete
    default: Incomplete
    model: Incomplete
    name: Incomplete
    primary_key: bool

class ManyToManyFieldAccessor(FieldAccessor):
    model: Incomplete
    rel_model: Incomplete
    through_model: Incomplete
    src_fk: Incomplete
    dest_fk: Incomplete
    def __init__(self, model, field, name) -> None: ...
    def __get__(self, instance, instance_type=None, force_query: bool = False): ...
    def __set__(self, instance, value) -> None: ...

class ManyToManyField(MetaField):
    accessor_class: Incomplete
    rel_model: Incomplete
    backref: Incomplete
    def __init__(
        self,
        model,
        backref=None,
        through_model=None,
        on_delete=None,
        on_update=None,
        prevent_unsaved: bool = True,
        _is_backref: bool = False,
    ) -> None: ...
    def bind(self, model, name, set_attribute: bool = True) -> None: ...
    def get_models(self) -> list[Incomplete]: ...

    @property
    def through_model(self): ...
    @through_model.setter
    def through_model(self, value) -> None: ...

    def get_through_model(self): ...

class VirtualField(MetaField):
    field_class: Incomplete
    field_instance: Incomplete
    def __init__(self, field_class=None, *args, **kwargs) -> None: ...
    def db_value(self, value): ...
    def python_value(self, value): ...
    model: Incomplete
    column_name: Incomplete
    def bind(self, model, name, set_attribute: bool = True) -> None: ...

class CompositeKey(MetaField):
    sequence: Incomplete
    field_names: Incomplete
    def __init__(self, *field_names) -> None: ...
    @property
    def safe_field_names(self): ...
    def __get__(self, instance, instance_type=None): ...
    def __set__(self, instance, value) -> None: ...
    def __eq__(self, other) -> Expression | bool: ...  # type: ignore[override]
    def __ne__(self, other) -> Expression | bool: ...  # type: ignore[override]
    def __hash__(self) -> int: ...
    def __sql__(self, ctx): ...
    model: Incomplete
    column_name: Incomplete
    def bind(self, model, name, set_attribute: bool = True) -> None: ...

class _SortedFieldList:
    __slots__ = ("_keys", "_items")
    def __init__(self) -> None: ...
    def __getitem__(self, i): ...
    def __iter__(self): ...
    def __contains__(self, item) -> bool: ...
    def index(self, field) -> int: ...
    def insert(self, item) -> None: ...
    def remove(self, item) -> None: ...

class SchemaManager:
    model: Incomplete
    context_options: Incomplete
    def __init__(self, model, database=None, **context_options) -> None: ...

    @property
    def database(self): ...
    @database.setter
    def database(self, value) -> None: ...

    def create_table(self, safe: bool = True, **options) -> None: ...
    def create_table_as(self, table_name, query, safe: bool = True, **meta) -> None: ...
    def drop_table(self, safe: bool = True, **options) -> None: ...
    def truncate_table(self, restart_identity: bool = False, cascade: bool = False) -> None: ...
    def create_indexes(self, safe: bool = True) -> None: ...
    def drop_index(self, field=None, index=None, safe: bool = True): ...
    def drop_indexes(self, safe: bool = True) -> None: ...
    def create_sequence(self, field) -> None: ...
    def drop_sequence(self, field) -> None: ...
    def create_foreign_key(self, field) -> None: ...
    def create_sequences(self) -> None: ...
    def create_all(self, safe: bool = True, **table_options) -> None: ...
    def drop_sequences(self) -> None: ...
    def drop_all(self, safe: bool = True, drop_sequences: bool = True, **options) -> None: ...

class Metadata:
    model: Incomplete
    database: Incomplete
    fields: Incomplete
    columns: Incomplete
    combined: Incomplete
    sorted_fields: Incomplete
    sorted_field_names: Incomplete
    defaults: Incomplete
    name: Incomplete
    table_function: Incomplete
    legacy_table_names: Incomplete
    table_name: Incomplete
    indexes: Incomplete
    constraints: Incomplete
    primary_key: Incomplete
    composite_key: Incomplete
    only_save_dirty: Incomplete
    depends_on: Incomplete
    table_settings: Incomplete
    without_rowid: Incomplete
    strict_tables: Incomplete
    temporary: Incomplete
    refs: Incomplete
    backrefs: Incomplete
    model_refs: Incomplete
    model_backrefs: Incomplete
    manytomany: Incomplete
    options: Incomplete
    def __init__(
        self,
        model,
        database=None,
        table_name=None,
        indexes=None,
        primary_key=None,
        constraints=None,
        schema: str | None = None,
        only_save_dirty: bool = False,
        depends_on=None,
        options=None,
        db_table=None,
        table_function=None,
        table_settings=None,
        without_rowid: bool = False,
        temporary: bool = False,
        strict_tables=None,
        legacy_table_names: bool = True,
        **kwargs,
    ) -> None: ...
    def make_table_name(self) -> str: ...
    def model_graph(
        self, refs: bool = True, backrefs: bool = True, depth_first: bool = True
    ) -> list[tuple[Incomplete, Incomplete, Incomplete]]: ...
    def add_ref(self, field) -> None: ...
    def remove_ref(self, field) -> None: ...
    def add_manytomany(self, field) -> None: ...
    def remove_manytomany(self, field) -> None: ...

    @property
    def table(self) -> Table: ...
    @table.deleter
    def table(self) -> None: ...

    @property
    def schema(self): ...
    @schema.setter
    def schema(self, value) -> None: ...

    @property
    def entity(self) -> Entity: ...
    def get_rel_for_model(self, model) -> tuple[list[Incomplete], list[Incomplete]]: ...
    def add_field(self, field_name, field, set_attribute: bool = True) -> None: ...
    def remove_field(self, field_name) -> None: ...
    auto_increment: Incomplete
    def set_primary_key(self, name, field) -> None: ...
    def get_primary_keys(self): ...
    def get_default_dict(self): ...
    def fields_to_index(self) -> list[Incomplete]: ...
    def set_database(self, database) -> None: ...
    def set_table_name(self, table_name) -> None: ...

class SubclassAwareMetadata(Metadata):
    models: Incomplete
    def __init__(self, model, *args, **kwargs) -> None: ...
    def map_models(self, fn) -> None: ...

class DoesNotExist(Exception): ...

class ModelBase(type):
    inheritable: set[str]
    def __new__(cls, name, bases, attrs): ...
    def __iter__(self): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value) -> None: ...
    def __delitem__(self, key) -> None: ...
    def __contains__(self, key) -> bool: ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    def __nonzero__(self) -> bool: ...
    def __sql__(self, ctx): ...

class _BoundModelsContext(_callable_context_manager):
    models: Incomplete
    database: Incomplete
    bind_refs: Incomplete
    bind_backrefs: Incomplete
    def __init__(self, models, database, bind_refs, bind_backrefs) -> None: ...
    def __enter__(self): ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class Model(metaclass=ModelBase):
    __data__: Incomplete
    __rel__: Incomplete
    _meta: Metadata
    def __init__(self, *args, **kwargs) -> None: ...
    @classmethod
    def validate_model(cls) -> None: ...
    @classmethod
    def alias(cls, alias=None) -> ModelAlias: ...
    @classmethod
    def select(cls, *fields) -> ModelSelect: ...
    @classmethod
    def update(cls, data=None, /, **update) -> ModelUpdate: ...
    @classmethod
    def insert(cls, data=None, /, **insert) -> ModelInsert: ...
    @classmethod
    def insert_many(cls, rows, fields=None) -> ModelInsert: ...
    @classmethod
    def insert_from(cls, query, fields) -> ModelInsert: ...
    @classmethod
    def replace(cls, data=None, /, **insert): ...
    @classmethod
    def replace_many(cls, rows, fields=None): ...
    @classmethod
    def raw(cls, sql, *params) -> ModelRaw: ...
    @classmethod
    def delete(cls) -> ModelDelete: ...
    @classmethod
    def create(cls, **query) -> Self: ...
    @classmethod
    def bulk_create(cls, model_list, batch_size=None) -> None: ...
    @classmethod
    def bulk_update(cls, model_list, fields, batch_size=None): ...
    @classmethod
    def noop(cls) -> NoopModelSelect: ...
    @classmethod
    def get(cls, *query, **filters): ...
    @classmethod
    def get_or_none(cls, *query, **filters): ...
    @classmethod
    def get_by_id(cls, pk): ...
    @classmethod
    def set_by_id(cls, key, value): ...
    @classmethod
    def delete_by_id(cls, pk): ...
    @classmethod
    def get_or_create(cls, **kwargs): ...
    @classmethod
    def filter(cls, *dq_nodes, **filters): ...
    def get_id(self): ...
    def save(self, force_insert: bool = False, only=None): ...
    def is_dirty(self) -> bool: ...
    @property
    def dirty_fields(self) -> list[Incomplete]: ...
    @property
    def dirty_field_names(self) -> list[Incomplete]: ...
    def dependencies(self, search_nullable: bool = True, exclude_null_children: bool = False) -> Generator[Incomplete]: ...
    def delete_instance(self, recursive: bool = False, delete_nullable: bool = False): ...
    def __hash__(self) -> int: ...
    def __eq__(self, other) -> Expression | bool: ...  # type: ignore[override]
    def __ne__(self, other) -> Expression | bool: ...  # type: ignore[override]
    def __sql__(self, ctx): ...
    @classmethod
    def bind(cls, database, bind_refs: bool = True, bind_backrefs: bool = True, _exclude=None) -> bool: ...
    @classmethod
    def bind_ctx(cls, database, bind_refs: bool = True, bind_backrefs: bool = True) -> _BoundModelsContext: ...
    @classmethod
    def table_exists(cls): ...
    @classmethod
    def create_table(cls, safe: bool = True, **options) -> None: ...
    @classmethod
    def drop_table(cls, safe: bool = True, drop_sequences: bool = True, **options) -> None: ...
    @classmethod
    def truncate_table(cls, **options) -> None: ...
    @classmethod
    def index(cls, *fields, **kwargs) -> ModelIndex: ...
    @classmethod
    def add_index(cls, *fields, **kwargs) -> None: ...

class ModelAlias(Node):
    def __init__(self, model, alias=None) -> None: ...
    def __getattr__(self, attr: str): ...
    def __setattr__(self, attr: str, value) -> None: ...
    def get_field_aliases(self) -> list[Incomplete]: ...
    def select(self, *selection) -> ModelSelect: ...
    def __call__(self, **kwargs): ...
    def __sql__(self, ctx): ...

class FieldAlias(Field):
    source: Incomplete
    model: Incomplete
    field: Incomplete
    def __init__(self, source, field) -> None: ...
    @classmethod
    def create(cls, source, field): ...
    def clone(self) -> FieldAlias: ...
    def adapt(self, value): ...
    def python_value(self, value): ...
    def db_value(self, value): ...
    def __getattr__(self, attr: str): ...
    def __sql__(self, ctx): ...

class _ModelQueryHelper:
    default_row_type: Incomplete
    def __init__(self, *args, **kwargs) -> None: ...
    def objects(self, constructor=None) -> Self: ...
    def models(self) -> Self: ...

class ModelRaw(_ModelQueryHelper, RawQuery):  # type: ignore[misc]
    model: Incomplete
    def __init__(self, model, sql, params, **kwargs) -> None: ...
    def get(self): ...

class BaseModelSelect(_ModelQueryHelper):
    def union_all(self, rhs) -> ModelCompoundSelectQuery: ...
    __add__ = union_all
    def union(self, rhs) -> ModelCompoundSelectQuery: ...
    __or__ = union
    def intersect(self, rhs) -> ModelCompoundSelectQuery: ...
    __and__ = intersect
    def except_(self, rhs) -> ModelCompoundSelectQuery: ...
    __sub__ = except_
    def __iter__(self): ...
    def prefetch(self, *subqueries): ...
    def get(self, database=None): ...
    def get_or_none(self, database=None): ...
    def group_by(self, *columns) -> Self: ...

class ModelCompoundSelectQuery(BaseModelSelect, CompoundSelectQuery):  # type: ignore[misc]
    model: Incomplete
    def __init__(self, model, *args, **kwargs) -> None: ...

class ModelSelect(BaseModelSelect, Select):  # type: ignore[misc]
    model: Incomplete
    def __init__(self, model, fields_or_models, is_default: bool = False) -> None: ...
    def clone(self) -> Self: ...
    def select(self, *fields_or_models): ...
    def select_extend(self, *columns): ...
    def switch(self, ctx=None) -> Self: ...
    def join(self, dest, join_type="INNER JOIN", on=None, src=None, attr=None) -> Self: ...  # type: ignore[override]
    def left_outer_join(self, dest, on=None, src=None, attr=None) -> Self: ...  # type: ignore[override]
    def join_from(self, src, dest, join_type="INNER JOIN", on=None, attr=None) -> Self: ...
    def ensure_join(self, lm, rm, on=None, **join_kwargs): ...
    def convert_dict_to_node(self, qdict) -> tuple[list[Incomplete], list[Incomplete]]: ...
    def filter(self, *args, **kwargs): ...
    def create_table(self, name, safe: bool = True, **meta): ...
    def __sql_selection__(self, ctx, is_subquery: bool = False): ...

class NoopModelSelect(ModelSelect):
    def __sql__(self, ctx): ...

class _ModelWriteQueryHelper(_ModelQueryHelper):
    model: Incomplete
    def __init__(self, model, *args, **kwargs) -> None: ...
    def returning(self, *returning): ...

class ModelUpdate(_ModelWriteQueryHelper, Update): ...  # type: ignore[misc]

class ModelInsert(_ModelWriteQueryHelper, Insert):  # type: ignore[misc]
    default_row_type: Incomplete
    def __init__(self, *args, **kwargs) -> None: ...
    def returning(self, *returning): ...
    def get_default_data(self): ...
    def get_default_columns(self): ...

class ModelDelete(_ModelWriteQueryHelper, Delete): ...  # type: ignore[misc]

class ManyToManyQuery(ModelSelect):
    def __init__(self, instance, accessor, rel, *args, **kwargs) -> None: ...
    def add(self, value, clear_existing: bool = False) -> None: ...
    def remove(self, value): ...
    def clear(self): ...

class BaseModelCursorWrapper(DictCursorWrapper):
    model: Incomplete
    select: Incomplete
    ncols: int
    columns: list[str]
    converters: list[Incomplete]
    fields: list[Incomplete]
    no_convert: list[int]
    convert: list[int]
    def __init__(self, cursor, model, columns) -> None: ...
    def initialize(self) -> None: ...
    def process_row(self, row): ...

class ModelDictCursorWrapper(BaseModelCursorWrapper):
    unique_columns: list[str]
    def initialize(self) -> None: ...
    def process_row(self, row) -> dict[str, Incomplete]: ...

class ModelTupleCursorWrapper(BaseModelCursorWrapper):
    constructor: Incomplete
    def process_row(self, row) -> tuple[Incomplete, ...]: ...  # type: ignore[override]

class ModelNamedTupleCursorWrapper(ModelTupleCursorWrapper):
    impl: Incomplete
    constructor: Incomplete
    def initialize(self) -> None: ...

class ModelObjectCursorWrapper(ModelDictCursorWrapper):
    constructor: Incomplete
    is_model: Incomplete
    identifiers: list[str]
    def __init__(self, cursor, model, select, constructor) -> None: ...
    def initialize(self) -> None: ...
    def process_row(self, row): ...

class ModelCursorWrapper(BaseModelCursorWrapper):
    from_list: Incomplete
    joins: Incomplete
    dicts: bool
    def __init__(self, cursor, model, select, from_list, joins, dicts: bool = False) -> None: ...
    key_to_constructor: Incomplete
    src_is_dest: dict[Incomplete, Incomplete]
    src_to_dest: list[tuple[Incomplete, Incomplete, Incomplete, bool, Incomplete, bool]]
    column_keys: list[Incomplete]
    def initialize(self) -> None: ...
    def process_row(self, row): ...

@type_check_only
class _PrefetchQuery(NamedTuple):
    query: Incomplete
    fields: Incomplete
    is_backref: bool
    rel_models: Incomplete
    field_to_name: Incomplete
    model: Incomplete

class PrefetchQuery(_PrefetchQuery):
    def __new__(
        cls, query, fields=None, is_backref: bool | None = None, rel_models=None, field_to_name=None, model=None
    ) -> Self: ...
    def populate_instance(self, instance, id_map) -> None: ...
    def store_instance(self, instance, id_map) -> None: ...

def prefetch(sq, *subqueries): ...

__all__ = [
    "AnyField",
    "AsIs",
    "AutoField",
    "BareField",
    "BigAutoField",
    "BigBitField",
    "BigIntegerField",
    "BinaryUUIDField",
    "BitField",
    "BlobField",
    "BooleanField",
    "Case",
    "Cast",
    "CharField",
    "Check",
    "chunked",
    "Column",
    "CompositeKey",
    "Context",
    "Database",
    "DatabaseError",
    "DatabaseProxy",
    "DataError",
    "DateField",
    "DateTimeField",
    "DecimalField",
    "Default",
    "DeferredForeignKey",
    "DeferredThroughModel",
    "DJANGO_MAP",
    "DoesNotExist",
    "DoubleField",
    "DQ",
    "Entity",
    "EXCLUDED",
    "Field",
    "FixedCharField",
    "FloatField",
    "fn",
    "ForeignKeyField",
    "IdentityField",
    "ImproperlyConfigured",
    "Index",
    "IntegerField",
    "IntegrityError",
    "InterfaceError",
    "InternalError",
    "IPField",
    "JOIN",
    "ManyToManyField",
    "Model",
    "ModelIndex",
    "MySQLDatabase",
    "NotSupportedError",
    "OP",
    "OperationalError",
    "PostgresqlDatabase",
    "PrimaryKeyField",
    "prefetch",
    "PREFETCH_TYPE",
    "ProgrammingError",
    "Proxy",
    "QualifiedNames",
    "SchemaManager",
    "SmallIntegerField",
    "Select",
    "SQL",
    "SqliteDatabase",
    "Table",
    "TextField",
    "TimeField",
    "TimestampField",
    "Tuple",
    "UUIDField",
    "Value",
    "ValuesList",
    "Window",
]



from _typeshed import Incomplete

logger: Incomplete

class Metrics:
    def __init__(self, default_config=None, reporters=None, enable_expiration: bool = False) -> None: ...
    @property
    def config(self): ...
    @property
    def metrics(self): ...
    def metric_name(self, name, group, description: str = "", tags=None): ...
    def get_sensor(self, name): ...
    def sensor(self, name, config=None, inactive_sensor_expiration_time_seconds=..., parents=None): ...
    def remove_sensor(self, name) -> None: ...
    def add_metric(self, metric_name, measurable, config=None) -> None: ...
    def remove_metric(self, metric_name): ...
    def add_reporter(self, reporter) -> None: ...
    def register_metric(self, metric) -> None: ...

    class ExpireSensorTask:
        @staticmethod
        def run(metrics) -> None: ...

    def close(self) -> None: ...
